package com.seitenbau.testing.instrumentation.generator;

import com.seitenbau.testing.instrumentation.IRecorderGenerator;
import com.seitenbau.testing.instrumentation.generator.impl.AspectJGenerator;
import com.seitenbau.testing.shared.config.RecorderConfiguration;
import com.seitenbau.testing.shared.helper.ReflectionHelper;

/**
 * Factory zur Instanzierung der Generatoren
 */
public final class RecorderGeneratorFactory
{

  /** Instanzierung ist nicht erlaubt */
  private RecorderGeneratorFactory()
  {
  }

  /**
   * Erzeugt den ber das Konfigurationsmodell spezifizierten
   * RecorderGenerator. Falls dieser nicht erzeugbar ist, wird der
   * DefaultGenerator() genutzt.
   *
   * @param config Konfigurations, welche den RecoderGenerator
   *        spezifiziert.
   *
   * @return Die konkrete Instanz, war keine konfiguriert, oder diese
   *         nicht instanzierbar, wird der DefaultGenerator
   *         zurckgeliefert.
   */
  public static IRecorderGenerator createRecorderGenerator(
      RecorderConfiguration config)
  {
    if (config != null )
    {
      String clazz = config.getRecorderGenerator();
      Object obj = ReflectionHelper.createInstanceSilent(clazz);
      if (obj instanceof IRecorderGenerator)
      {
        return (IRecorderGenerator) obj;
      }
    }
    // Falls kein alternativer Generator erzeugt werden konnte:
    return new AspectJGenerator();
  }

}
