package com.seitenbau.testing.instrumentation.generator.impl;

import java.util.List;

import com.seitenbau.testing.instrumentation.IRecorderGenerator;
import com.seitenbau.testing.instrumentation.generator.RecorderParams;
import com.seitenbau.testing.shared.config.IListenerConfiguration;
import com.seitenbau.testing.shared.config.MethodConfiguration;
import com.seitenbau.testing.shared.config.RecorderConfiguration;
import com.seitenbau.testing.shared.config.TracerConfiguration;
import com.seitenbau.testing.shared.locator.IRecorderPathLocator;
import com.seitenbau.testing.shared.locator.RecorderPathLocator;
import com.seitenbau.testing.shared.templateengine.ITemplateEngine;
import com.seitenbau.testing.shared.templateengine.ITemplateParams;
import com.seitenbau.testing.shared.templateengine.TemplateEngine;
import com.seitenbau.testing.shared.writer.IWriter;
import com.seitenbau.testing.shared.writer.LoggerWriter;
import com.seitenbau.testing.templates.Record_AspectGenerator;


public class AspectJGenerator implements IRecorderGenerator
{ 

  /**
   * Erzeugt alle ntigen Artefakte zur Instrumentation des Ziel-Codes
   * durch AspectJ.
   *
   * Dies ist eine Aspeckt, welcher durch JetTemplateEngine(
   * ITemplateConstants.TEMPLATE_RECORD_ASPECT ) erzeugt wird.
   * Auerdem wird das
   *
   */
  public void createRecorderArtifacts(RecorderConfiguration configuration,
      String targetSrcRootFolder)
  {
    createArtifacts( configuration,targetSrcRootFolder,targetSrcRootFolder);
  }

  public void createRecorderArtifacts(RecorderConfiguration configuration,
      String targetSrcRootFolder, String targetResourceRootFolder)
  {
      createArtifacts( configuration,targetSrcRootFolder,targetResourceRootFolder);
  }

  private void createArtifacts(RecorderConfiguration configuration,
      String srcRoot,String resourceRoot)
  {

    MethodConfiguration first = configuration.getMethodConfiguration().get(0);
    String pacakgeName = first.getPackageName();
    String prefix=configuration.getBaseName();


    String classname= first.getClassName();
    IRecorderPathLocator locator=new RecorderPathLocator(
        prefix+classname,srcRoot,resourceRoot,pacakgeName);


    RecorderParams recParams = new RecorderParams(configuration, locator);

    ITemplateEngine eingine = createNewJetEngine(recParams);
    eingine.generateToFile(new Record_AspectGenerator(), recParams);

    String file = locator.getResourceFile(".record.xml");

    recParams.getConfig().saveFile( file);

    LoggerWriter writer = new LoggerWriter();
    printHelpMessage(configuration, writer);

  }

  private void printHelpMessage(RecorderConfiguration config, IWriter writer)
  {
    MethodConfiguration oneMethod = config.getMethodConfiguration().get(0);
    TracerConfiguration tracer = oneMethod.getTracer();
    if (tracer != null)
    {
      if (tracer instanceof IPrintHelpMessage)
      {
        printHelp((IPrintHelpMessage) tracer, writer);
      }
      List<IListenerConfiguration> listeners = tracer.getListeners();
      if (listeners != null)
      {
        for (IListenerConfiguration aListener : listeners)
        {
          if (aListener instanceof IPrintHelpMessage)
          {
            printHelp((IPrintHelpMessage) aListener, writer);
          }
        }
      }
    }
  }

  private void printHelp(IPrintHelpMessage listener, IWriter writer)
  {
    listener.printHelpMessage(writer);
  }

  protected ITemplateEngine createNewJetEngine(ITemplateParams params)
  {
    TemplateEngine te = new TemplateEngine();
    // te.setTemplateParams(params);
    return te;
  }

}
