package com.seitenbau.testing.service;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import com.seitenbau.testing.shared.config.MethodConfiguration;
import com.seitenbau.testing.shared.config.RecorderConfiguration;
import com.seitenbau.testing.shared.config.RepositoryConfig;
import com.seitenbau.testing.shared.dao.DaoFactory;
import com.seitenbau.testing.shared.dao.IDaoFactory;
import com.seitenbau.testing.shared.helper.LoggerHelper;
import com.seitenbau.testing.shared.model.RecordedMethodConfiguration;
import com.seitenbau.testing.shared.model.RecordedTest;
import com.seitenbau.testing.shared.model.RecordedTestCase;


public class RepositoryService
{

  /**
   * Entfernt den bergebenen Test aus den Daten.
   *
   * @param config Konfigurations Modell, zum Finden der
   *        Datenbank-Verbindung.
   *
   * @param test Der zu entfernende Tests
   *
   */
  public static void removeRecordedTest(RecorderConfiguration config,
      RecordedTest test)
  {
    try
    {
      DaoFactory.create(config).getRecordedTestDao().delete(test);
    }
    catch (Throwable t)
    {
      t.printStackTrace();
    }

  }

  /**
   * Entfernen des angegebenen RecordedTestCase & der enthaltenen
   * Tests aus der Datenbank
   *
   * @param config
   * @param testCase
   */
  public static void removeRecordedTestCase(RecorderConfiguration config,
      RecordedTestCase testCase)
  {
    try
    {
      DaoFactory.create(config).getRecordedTestCaseDao().delete(testCase);
    }
    catch (Throwable t)
    {
      t.printStackTrace();
    }
  }

  /**
   * Liefert alle zur gegenen Konfiguration vorhandenen
   * RecordedMethodConfigurations
   *
   *
   * @param configFileName Dateisystem Pfad zur Konfigurationsdatei
   *
   * @return Liste aller in der Datenbank gefundenen
   *         RecordedMethodConfiguration Objekte
   *
   */
  public static List<RecordedMethodConfiguration> getRecordedMethodConfigurations(
      String configFileName)
  {
    RecorderConfiguration cfg = RecorderConfiguration.loadFile(configFileName);

    return getRecordedMethodConfigurations(cfg);
  }

  /**
   * Liefert alle zur gegenen Konfiguration vorhandenen
   * RecordedMethodConfigurations
   *
   *
   * @param cfg RecorderConfiguration Modell
   *
   * @return Liste aller in der Datenbank gefundenen
   *         RecordedMethodConfiguration Objekte
   *
   */
  public static List<RecordedMethodConfiguration> getRecordedMethodConfigurations(
      RecorderConfiguration cfg)
  {
    IDaoFactory dao = DaoFactory.create(cfg);
    if (dao != null)
    {
      List<RecordedMethodConfiguration> rcList = new LinkedList<RecordedMethodConfiguration>();
      getAllRecordConfig(rcList, cfg, dao);
      return rcList;
    }
    else
    {
      return null;
    }

  }

  /**
   * Liste aller aufgezeichneten RecordedTestCase Element einer
   * RecorderConfiguration
   *
   * @param cfg
   * @return
   */
  public static List<RecordedTestCase> getRecordedTestCases(
      RecorderConfiguration cfg)
  {
    IDaoFactory dao = DaoFactory.create(cfg);

    if (dao != null)
    {
      List<RecordedTestCase> rcList = new LinkedList<RecordedTestCase>();

      if (cfg.getMethodConfiguration() != null)
      {
        for (MethodConfiguration mcfg : cfg.getMethodConfiguration())
        {
          RecordedMethodConfiguration rc = dao.getRecordConfigDao().findBy(
              mcfg.getPackageName(), mcfg.getClassName(), mcfg.getMethodName(),
              mcfg.getReturnType(), mcfg.getParameters());
          if (rc != null)
          {
            rcList.addAll(rc.getRecordedTestCases());
          }
        }
      }
      return rcList;
    }
    else
    {
      return null;
    }
  }

  /**
   *
   * Interne Methode zum Aufbau einer RecordedMethodConfiguration
   * Liste
   *
   * @param rcList
   * @param cfg
   * @param dao
   */
  private static void getAllRecordConfig(
      List<RecordedMethodConfiguration> rcList, RecorderConfiguration cfg,
      IDaoFactory dao)
  {
    if (cfg.getMethodConfiguration() != null)
    {
      for (MethodConfiguration mcfg : cfg.getMethodConfiguration())
      {
        RecordedMethodConfiguration rc = dao.getRecordConfigDao().findBy(
            mcfg.getPackageName(), mcfg.getClassName(), mcfg.getMethodName(),
            mcfg.getReturnType(), mcfg.getParameters());

        if (rc != null)
        {
          rcList.add(rc);
        }
      }
    }
  }

  public static List<RecordedMethodConfiguration> getTestData(
      MethodConfiguration methodConfiguration, RepositoryConfig repository)
  {
    IDaoFactory dao = DaoFactory.create(repository);

    RecordedMethodConfiguration rc = dao.getRecordConfigDao().findBy(
        methodConfiguration.getPackageName(),
        methodConfiguration.getClassName(),
        methodConfiguration.getMethodName(),
        methodConfiguration.getReturnType(),
        methodConfiguration.getParameters());

    List<RecordedMethodConfiguration> rcList = new ArrayList<RecordedMethodConfiguration>();
    if (rc != null)
    {
      rcList.add(rc);
    }
    if(rcList.isEmpty()) {
      LoggerHelper.logger.warning("No content found in Repository for specified Method.");
    }
    return rcList;
  }

}
