package com.seitenbau.testing.shared.aspects;

import com.seitenbau.testing.shared.tracer.TracerControl;
import com.seitenbau.testing.shared.helper.LoggerHelper;
import com.seitenbau.testing.shared.helper.ReflectionHelper;
import com.seitenbau.testing.shared.tracer.ITracer;
import com.seitenbau.testing.shared.tracer.NullTracer;

/**
 * Basic Aspect for Tecording Tests
 *
 * Following Annotations are supported:
 *
 * <li>@{@see IgnoreAspect}</li>
 *
 */
public abstract aspect AbstractBaseAspect extends AbstractBasePointcuts
{

  static
  {
    // LoggerHelper.logger.fine("AbstractBaseAspect::staticInitializer()");
  }

  abstract protected ITracer createAspectTracer();

  protected ITracer myAspectTracer = null;

  public AbstractBaseAspect()
  {
//    getAspectTracer();
  }

  /** Around the Method to observe */
  Object around()  : pc_CallToTrace()
	{
    getAspectTracer().targetMethodEntering(thisJoinPoint);

    Object theReturnValue = null;
    try
    {
      theReturnValue = proceed();
    }
    catch (Throwable t)
    {
      getAspectTracer().targetMethodThrowedException(thisJoinPoint, t);
      ReflectionHelper.throwException(t);
    }

    getAspectTracer().targetMethodProceeded(thisJoinPoint, theReturnValue);
    return theReturnValue;
  }

  /** Call to new Operations */
  Object around() : pc_allNewCalls()
	{
    getAspectTracer().subNewOperatorEntering(thisJoinPoint);
    Object theReturnValue = null;
    try
    {
      theReturnValue = proceed();
      getAspectTracer().subNewOperatorProceeded(thisJoinPoint, theReturnValue);
    }
    catch (Throwable t)
    {
      getAspectTracer().subNewOperatorThrowedException(thisJoinPoint, t);
      ReflectionHelper.throwException(t);

    }
    return theReturnValue;
  }

  /** Call to subsequent methods (including static methods) */
  Object around() : pc_allMethodCalls()
	{
    getAspectTracer().subMethodEntering(thisJoinPoint);

    Object theReturnValue = null;
    try
    {
      theReturnValue = proceed();
      getAspectTracer().subMethodProceeded(thisJoinPoint, theReturnValue);
    }
    catch (Throwable t)
    {
      getAspectTracer().subMethodThrowedException(thisJoinPoint, t);
      ReflectionHelper.throwException(t);
    }
    return theReturnValue;
  }

  /**
   * Get the current active ITracer if none set, till now a NullTracer
   * will be returned!
   */
  protected ITracer getAspectTracer()
  {
    if (myAspectTracer == null)
    {
      myAspectTracer = createNewTracer();
    }
    return myAspectTracer;
  }

  protected ITracer createNewTracer()
  {
    ITracer newTracer = new NullTracer();
    try
    {

      if (this.getClass().getAnnotation(IgnoreAspect.class) != null)
      {
//        System.out.println("IgnoreAspect Annotation used. Using NullTracer.");
        LoggerHelper.logger
            .fine("IgnoreAspect Annotation used. Using NullTracer.");
      }
      else if (!TracerControl.isTracerEnabled())
      {
//        System.out.println("TracerControl isTracerEnabled == false. Using NullTracer.");
        LoggerHelper.logger
            .fine("TracerControl isTracerEnabled == false. Using NullTracer.");
      }
      else
      {
        newTracer = createAspectTracer();
      }

    }
    catch (Throwable t)
    {
      t.printStackTrace();
      LoggerHelper.logger.severe("AbstractBaseAspect() Initialisation failed: "
          + t.toString());
    }
    return newTracer;
  }

  /**
   * Set the current active ITracer.
   */
  public void setAspectTracer(ITracer aspectTracer)
  {
    myAspectTracer = aspectTracer;
  }
}
