package com.seitenbau.testing.shared.aspects;

import static com.seitenbau.testing.shared.helper.LoggerHelper.logger;
import com.seitenbau.testing.shared.helper.ReflectionHelper;
import com.seitenbau.testing.shared.tracer.flow.IFlowControl;
/**
 * Basic Aspect to support mocking.
 * 
 * <b>Remark on: <i>don't repeat yourself</i>.</b> At the moment
 * many this class is a more or less copy of AbstractBaseAspect. This
 * is because the Aspect - proceed() Method is only acessible within
 * around(). At the moment I don't know how to 'export' this call to
 * an external Method.
 */
public abstract aspect AbstractMockAspect extends AbstractBasePointcuts {

	/** The delegated Tracer for flow control */
	private IFlowControl myFlowControl=null;

	/** Overwrite this to set the Concrete FlowControl Instance */
	abstract protected IFlowControl createFlowControl();

	protected IFlowControl getFlowControl() {
		if(myFlowControl==null) {
			myFlowControl=createFlowControl();
			if(myFlowControl==null) {
				logger.warning("AbstractMockAspect::getFlowControl() : myFlowControl is null, switching to proceed.");
			}
		}
		return myFlowControl;
	}

	protected void resetFlowControl() {
		myFlowControl=null;
	}
	/** Around the Method to observe */
	Object around() : pc_CallToTrace()  {
		resetFlowControl();
		IFlowControl flowControl=getFlowControl();
		if( flowControl==null) return proceed();

		flowControl.targetMethodEntering(thisJoinPoint);

		Object theReturnValue=null;
		Throwable aThrowedException=null;

		// Call the original code ...
		if( flowControl.targetMethodShouldProceed(thisJoinPoint) ) {
			try {
				theReturnValue=proceed();
				flowControl.targetMethodProceeded(thisJoinPoint,theReturnValue);
			} catch(Throwable t) {
				flowControl.targetMethodThrowedException(thisJoinPoint, t);
				// Remember, but don't throw it. The decision to throw or not
        // comes later.
				aThrowedException=t;
			}
		}
		// Decide if a Exception has to be thrown
		if( flowControl.targetMethodSchouldThrowException(thisJoinPoint,aThrowedException) ) {
			ReflectionHelper.throwException(
					flowControl.targetMethodThrowingException(thisJoinPoint, aThrowedException)
					);
		}

		// Finally we can return ...
		return flowControl.targetMethodReturningValue(thisJoinPoint, theReturnValue);
	}

	/** Call to new Operations */
	Object around() : pc_allNewCalls() {
		IFlowControl flowControl=getFlowControl();
		if( flowControl==null) return proceed();

		flowControl.subNewOperatorEntering(thisJoinPoint);

		Object theReturnValue=null;
		Throwable aThrowedException=null;
		// Call the original code ...
		if( flowControl.subNewOperatorShoudProceed(thisJoinPoint) ) {
			try {
				theReturnValue=proceed();
				flowControl.subNewOperatorProceeded(thisJoinPoint,theReturnValue);
			} catch(Throwable t) {
				flowControl.subNewOperatorThrowedException(thisJoinPoint, t);
				// Remember, but don't throw it. The decision to throw or not
        // comes later.
				aThrowedException=t;
			}
		}

		// Decide if a Exception has to be thrown
		if( flowControl.subNewOperatorSchouldThrowException(thisJoinPoint,aThrowedException) ) {
			ReflectionHelper.throwException(
					flowControl.subNewOperatorThrowingException(thisJoinPoint, aThrowedException)
					);
		}

		// Finally we can return ...
		return flowControl.subNewOperatorReturningValue(thisJoinPoint, theReturnValue);
	}

	/** Call to subsequent methods (including static methods) */
	Object around() : pc_allMethodCalls() {
		IFlowControl flowControl=getFlowControl();
		if( flowControl==null) return proceed();

		flowControl.subMethodEntering(thisJoinPoint);

		Object theReturnValue=null;
		Throwable aThrowedException=null;

		// Call the original code ...
		if( flowControl.subMethodShouldProceed(thisJoinPoint) ) {
			try {
				theReturnValue=proceed();
				flowControl.subMethodProceeded(thisJoinPoint,theReturnValue);
			} catch(Throwable t) {
				flowControl.subMethodThrowedException(thisJoinPoint, t);
				// Remember, but don't throw it. The decision to throw or not
        // comes later.
				aThrowedException=t;
			}
		}
		// Decide if a Exception has to be thrown
		if( flowControl.subMethodSchouldThrowException(thisJoinPoint,aThrowedException) ) {
			ReflectionHelper.throwException(
					flowControl.subMethodThrowingException(thisJoinPoint, aThrowedException)
					);
		}
		// Finally we can return ...
		return flowControl.subMethodReturningValue(thisJoinPoint, theReturnValue);
	}
}
