package com.seitenbau.testing.shared.config;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import com.seitenbau.testing.shared.helper.FileHelper;
import com.seitenbau.testing.shared.helper.TRConst;
import com.seitenbau.testing.shared.plugin.IXStreamSweetner;
import com.seitenbau.testing.shared.plugin.SBRecorderPlugins;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.annotations.XStreamImplicit;

/**
 * Wurzel-Elemet der Recorder Konfiguration.
 */
public class RecorderConfiguration
{

  /**
   * Einfacher Name. Dieser wird genutzt um mgliche Artefakte mit
   * einem gleichen Teil beginnen zu lassen.
   */
  @XmlField
  private String name = "DummyName";

  /**
   * Die Datenbank Konfiguration.
   */
  private RepositoryConfig Database = new RepositoryConfig();

  private String baseName;


  /**
   * Menge an Methoden Konfigurationen
   */
  @XStreamImplicit(itemFieldName = "Method")
  private List<MethodConfiguration> methodConfiguration = new Vector<MethodConfiguration>();

  private String recorderGenerator;

  /** Ldt die Konfiguration von einer XML-Datei. */
  public static RecorderConfiguration loadFile(String filename)
  {
    try
    {
      XStream xs = getXStream();
      return (RecorderConfiguration) FileHelper
          .readObjectFromFile(xs, filename);
    }
    catch (RuntimeException xe)
    {
      xe.printStackTrace();
      throw xe;
    }
  }

  /** Ldt die Konfiguration aus einem XML-InputStream */
  public static RecorderConfiguration loadFromStream(InputStream inStream)
  {
    if (inStream == null)
    {
      throw new IllegalArgumentException("Passed InputStream was null.");
    }
    try
    {
      // Thread.currentThread().setContextClassLoader(RecorderConfiguration.class.getClassLoader());
      return (RecorderConfiguration) FileHelper.readObjectFromStream(
          getXStream(), inStream);
    }
    catch (RuntimeException xe)
    {
      xe.printStackTrace();
      throw xe;
    }
  }

  //
  // /** Ldt die Konfiguration von einer XML-Datei. */
  // public void parseFile(String filename) {
  //
  // try {
  // DocumentBuilderFactory docBuildFactory = DocumentBuilderFactory
  // .newInstance();
  //
  // DocumentBuilder parser;
  // parser = docBuildFactory.newDocumentBuilder();
  //
  // Document document = parser.parse(new File(filename));
  //
  // parse(document.getFirstChild());
  //
  // } catch (Exception e) {
  // e.printStackTrace();
  // throw new IllegalArgumentException("Unable to parse file" + e);
  // }
  //
  // }

  // public void parse(Node configRoot) {
  // Field[] fields = getClass().getDeclaredFields();
  // for (int i = 0; i < fields.length; i++) {
  // Field field = fields[i];
  // if (field.getAnnotation(XmlField.class) != null) {
  // field.setAccessible(true);
  // Object value = readString(configRoot, "name");
  // try {
  // field.set(this, value);
  // } catch (IllegalArgumentException e) {
  // e.printStackTrace();
  // } catch (IllegalAccessException e) {
  // e.printStackTrace();
  // }
  // }
  // }
  // // name = readString (configRoot,"name");
  // // Global = readConfig
  // (configRoot,"Global","GlobalConfiguration");
  // //
  // methodConfiguration=readList(configRoot,"Method","MethodConfiguration");
  // }

  /**
   * Hilfs-Methode zum erzeugen&konfigurieren von XStream fr die XML
   * de/serialisierung
   */
  public static XStream getXStream()
  {
    XStream xs = new XStream();
    xs.alias("TestRecorderConfiguration", RecorderConfiguration.class);
    xs.autodetectAnnotations(true);
    if (TRConst.RECORDER_CONFIG_SWEETEN_XSTREAM_BY_PLUGINS)
    {
      ArrayList<IXStreamSweetner> plugs = SBRecorderPlugins
          .getPlugins(IXStreamSweetner.class);
      for (IXStreamSweetner plugin : plugs)
      {
        plugin.registerOnXStream(xs);
      }
    }
    return xs;
  }

  /**
   * Sepeichert die Konfiguration unter dem bergemenen Dateinamen als
   * XML Datei ab.
   *
   * @param filename Die Ziel-XML-Datei
   */
  public void saveFile(String filename)
  {
    XStream xs = getXStream();
    FileHelper.writeFile(filename, xs.toXML(this));
  }

  public RepositoryConfig getRepository()
  {
    if (Database == null)
    {
      Database = new RepositoryConfig();
    }
    return Database;
  }

  public void setRepository(RepositoryConfig configuration)
  {
    Database = configuration;
  }

  public String getName()
  {
    return name;
  }

  public void setName(String name)
  {
    this.name = name;
  }

  public List<MethodConfiguration> getMethodConfiguration()
  {
    return methodConfiguration;
  }

  public void setMethodConfiguration(
      List<MethodConfiguration> aMethodConfiguration)
  {
    methodConfiguration = aMethodConfiguration;
  }

  public void addMethodConfiguration(MethodConfiguration methodConfiguration)
  {
    getMethodConfiguration().add(methodConfiguration);
  }

  public void removeMethodConfiguration(MethodConfiguration aMethodconfiguration)
  {
    getMethodConfiguration().remove(aMethodconfiguration);
  }

  public String getBaseName()
  {
    if (baseName == null)
    {
      return "";
    }
    return baseName;
  }

  public void setBaseName(String baseName)
  {
    this.baseName = baseName;
  }

  public String getRecorderGenerator()
  {
    return recorderGenerator;
  }

}
