package com.seitenbau.testing.shared.config;

import java.util.Properties;

import com.thoughtworks.xstream.annotations.XStreamAlias;

/**
 * Konfigurations Modell fr die Datenbankverbindung.
 *
 * Enthlt alle ntigen Informationen ber die Verbindung zur
 * Datenbank.
 *
 */
public class RepositoryConfig
{

  /**
   * Datenbank connection URL
   */
  @XStreamAlias("DatabaseURL")
  private String myDatabaseURL = null;

  /**
   * Datenbank driver
   */
  @XStreamAlias("DriverClass")
  private String myDriverClass = null;

  /**
   * Datenbank Benutzername
   */
  @XStreamAlias("Username")
  private String myUsername = null;

  /**
   * Passwort fr den Datenbank Benutzer
   */
  @XStreamAlias("Password")
  private String myPassword = null;

  /**
   * Alternative DaoFactory, default ist die HibernateDaoFactory
   */
  @XStreamAlias("DatabaseFactory")
  private String myDatabaseFactory = null;

  /**
   * weitere Optionen
   *
   * Abhngig von der gewhlten DaoFactory
   */
  @XStreamAlias("Properties")
  private Properties myProperties = null;

  /**
   * Default Constructor.
   *
   * <p>The default Settings are a MySQL sbtestrecorder Database on localhost.
   * Same as the following:
   * <pre><code>
   * new RepositoryConfig(
   *     "com.mysql.jdbc.Driver",
   *     "jdbc:mysql://localhost/sbtestrecorder",
   *     "root",
   *     ""
   *   );
   */
  public RepositoryConfig()
  {
    // See hibernate.cfg.xml for the default Database connection
  }

  /**
   * Convenient Constructor to directly setup the Database connection parameters
   *
   * <p>For a default MySQL installation on localhost with a sbtestrecorder Database
   * a Constructor call looks like:
   * <pre><code>
   * new RepositoryConfig(
   *     "com.mysql.jdbc.Driver",
   *     "jdbc:mysql://localhost/sbtestrecorder",
   *     "root",
   *     ""
   *   );
   * </code></pre>
   *
   * @param databaseDriver
   *    The Database Driver. e.g. "com.mysql.jdbc.Driver" for a MySQL Server.
   *
   * @param databaseURL
   *    The Database URL. e.g. "jdbc:mysql://localhost/sbtestrecorder" for a MySQL
   *    Server on localhost with a Database sbtestrecorder.
   *
   * @param databaseUserName
   *    The Username for the Database user which can write into the Database. e.g.
   *    "root" in a default MySQL installation.
   *
   * @param databasePassword
   *    The Password for the given User. e.g. "" in a default MySQL installation.
   *
   */
  public RepositoryConfig(String databaseDriver, String databaseURL, String databaseUserName,
      String databasePassword)
  {
    setDriverClass(databaseDriver);
    setDatabaseURL(databaseURL);
    setUsername(databaseUserName);
    setPassword(databasePassword);
  }

  public String getDatabaseFactory()
  {
    return myDatabaseFactory;
  }

  public void setDatabaseFactory(String databaseDao)
  {
    myDatabaseFactory = databaseDao;
  }

  public String getDatabaseURL()
  {
    return myDatabaseURL;
  }

  public void setDatabaseURL(String databaseURL)
  {
    this.myDatabaseURL = databaseURL;
  }

  public String getUsername()
  {
    return myUsername;
  }

  public void setUsername(String username)
  {
    this.myUsername = username;
  }

  public String getPassword()
  {
    return myPassword;
  }

  public void setPassword(String password)
  {
    this.myPassword = password;
  }

  public Properties getProperties()
  {
    return myProperties;
  }

  public void setProperties(Properties someProperties)
  {
    this.myProperties = someProperties;
  }

  public String getDriverClass()
  {
    return myDriverClass;
  }

  public void setDriverClass(String databaseDriver)
  {
    this.myDriverClass = databaseDriver;
  }

}
