package com.seitenbau.testing.shared.config;

import static com.seitenbau.testing.shared.helper.LoggerHelper.logger;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import com.seitenbau.testing.shared.helper.FileHelper;
import com.seitenbau.testing.shared.model.BasicDBModel;
import com.seitenbau.testing.shared.model.RecordedMethodConfiguration;
import com.seitenbau.testing.shared.xstream.XStreamObjectSnapshotSweetener;
import com.thoughtworks.xstream.XStream;

public class TestConfiguration
{

	 private List<RecordedMethodConfiguration> testDataList;

	  private List<IXmlFilterConfiguration> filters;

	  private String myBaseName;

  public void writeTofile(String fileName)
  {
    String xml = getXStream().toXML(this);
    FileHelper.writeFile(fileName, xml);
  }

  private static XStream getXStream()
  {
    XStream xs = new XStream();
    xs.registerConverter(new XStreamObjectSnapshotSweetener());

    xs.omitField(BasicDBModel.class, "myID");
    xs.autodetectAnnotations(true);
    return xs;
  }

  public static TestConfiguration readFromStream(Class<?> clazz, String fileName)
  {
    InputStream is = clazz.getResourceAsStream(fileName);
    Object configData = getXStream().fromXML(is);
    if (configData instanceof TestConfiguration)
    {
      return (TestConfiguration) configData;
    }
    else
    {
      logger.severe("Unable to load TestConfiguation from Stream. " + clazz
          + " FileName: " + fileName);
      return null;
    }
  }

  public List<RecordedMethodConfiguration> getTestData()
  {
    return testDataList;
  }

  public void setTestData(List<RecordedMethodConfiguration> methodList)
  {
    this.testDataList = methodList;
  }

  public List<IXmlFilterConfiguration> getFilters()
  {
    if (filters == null)
    {
      filters = new ArrayList<IXmlFilterConfiguration>();
    }
    return filters;
  }

  public void setFilters(List<IXmlFilterConfiguration> filters)
  {
    this.filters = filters;
  }

  public void addFilter(IXmlFilterConfiguration filter)
  {
    getFilters().add(filter);
  }

  public void setBaseName(String string)
  {
    myBaseName = string;

  }

  public String getBaseName()
  {
    return myBaseName;
  }
}
