package com.seitenbau.testing.shared.config;

import java.util.List;
import java.util.Properties;
import java.util.Vector;

import com.seitenbau.testing.shared.tracer.ITracer;
import com.seitenbau.testing.shared.tracer.ITracerListener;
import com.thoughtworks.xstream.annotations.XStreamImplicit;

/**
 * Konfiguration des Tracers
 */
public class TracerConfiguration
{

  private String TracerClass = null;

  /** Zustzliche Optionen fr den spezifischen Tracer */
  private Properties Properties = null;

  @XStreamImplicit(itemFieldName = "Listener")
  private List<IListenerConfiguration> Listeners = new Vector<IListenerConfiguration>();

  private MethodConfiguration myMethodConfiguration;

  public TracerConfiguration()
  {

  }

  public TracerConfiguration(Class<? extends ITracer> clazz)
  {
    setTracerClass(clazz);
  }

  public MethodConfiguration getMethodConfiguration()
  {
    return myMethodConfiguration;
  }

  public void setMethodConfiguration(MethodConfiguration methodConfiguration)
  {
    myMethodConfiguration = methodConfiguration;
  }

  public List<IListenerConfiguration> getListeners()
  {
    return Listeners;
  }

  public void setListeners(List<IListenerConfiguration> someListeners)
  {
    Listeners = someListeners;
  }

  public void addListener(IListenerConfiguration aListener)
  {
    getListeners().add(aListener);
  }

  public void removeListener(ListenerConfiguration aListener)
  {
    if (aListener != null)
    {
      getListeners().remove(aListener);
    }

  }

  public String getTracerClass()
  {
    return TracerClass;
  }

  public Properties getProperties()
  {
    if (Properties == null)
    {
      Properties = new Properties();
    }
    return Properties;
  }

  public void setProperties(Properties someProperties)
  {
    this.Properties = someProperties;
  }

  public void setTracerClass(Class<? extends ITracer> clazz)
  {
    if(clazz!=null)
    {
      TracerClass = clazz.getCanonicalName();
    }
    else
    {
      TracerClass = null;
    }
  }

  protected void setTracerClass(String recorderClass)
  {
    TracerClass = recorderClass;
  }

  /**
   * Convenient method for adding a Tracer.
   *
   * <p>Indirectly creates an ListenerConfiguration which is used to wrap the
   * passed Class object.
   *
   * @param clazzOfTracerListener
   *    The concrete Tracer Listener you want to add. Wrapped by a ListenerConfiguration
   *    which is then returned.
   *
   * @return
   *    The concrete TracerConfiguration used to wrap this Tracer
   *
   */
  public ListenerConfiguration addListener(Class<? extends ITracerListener> clazzOfTracerListener)
  {
    ListenerConfiguration listenerConfig = new ListenerConfiguration(clazzOfTracerListener);
    addListener( listenerConfig );
    return listenerConfig;
  }
}
