package com.seitenbau.testing.shared.dao;

import java.util.HashMap;

import com.seitenbau.testing.shared.config.RecorderConfiguration;
import com.seitenbau.testing.shared.config.RepositoryConfig;
import com.seitenbau.testing.shared.dao.hibernate.HibernateDaoFactory;
import com.seitenbau.testing.shared.helper.ReflectionHelper;


/**
 * Factory zum erzeugen der DaoFactory
 */
public final class DaoFactory
{

  /** Keine Instanzen erlaubt */
  private DaoFactory()
  {
  }

  private static HashMap<RecorderConfiguration, IDaoFactory> myHash = new HashMap<RecorderConfiguration, IDaoFactory>();

  public static IDaoFactory create(RecorderConfiguration config)
  {

    IDaoFactory factory = myHash.get(config);
    if (factory != null)
    {
      return factory;
    }
    if (config != null)
    {
      factory = createFrom(config.getRepository());
    }
    else
    {
      factory = new HibernateDaoFactory();
      factory.setConfiguration(null);
    }

    myHash.put(config, factory);
    return factory;
  }

  private static IDaoFactory createFrom(RepositoryConfig dbcfg)
  {
    IDaoFactory factory;
    if (dbcfg == null)
    {
      factory = new HibernateDaoFactory();
    }
    else
    {
      Object obj = ReflectionHelper.createInstanceSilent(dbcfg
          .getDatabaseFactory());
      if (obj instanceof IDaoFactory)
      {
        factory = (IDaoFactory) obj;
      }
      else
      {
        factory = new HibernateDaoFactory();
      }
    }
    factory.setConfiguration(dbcfg);
    return factory;
  }

  public static IDaoFactory create(RepositoryConfig repository)
  {
    return createFrom(repository);
  }
}
