package com.seitenbau.testing.shared.dao.hibernate;

import java.util.List;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

abstract public class HibernateDao<GEN_TYPE>
{

  private HibernateDaoFactory myHibernateDaoFactory = null;

  abstract protected Class<?> getClassType();

  protected void setHibernateDaoFactory(HibernateDaoFactory hDaoFactory)
  {
    if (hDaoFactory == null)
    {
      throw new IllegalArgumentException("HibernateDaoFactory cannot be null.");
    }
    myHibernateDaoFactory = hDaoFactory;
  }

  protected String getClassTable()
  {
    return getClassType().getSimpleName();
  }

  @SuppressWarnings("unchecked")
  public GEN_TYPE findByID(int id)
  {
    Session session = getSession();
    Transaction trans = beginTransaction(session);
    GEN_TYPE returnObj = null;
    try
    {
      returnObj = (GEN_TYPE) session.load(getClassType(), id);
      trans.rollback();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      trans.rollback();
    }
    return returnObj;
  }

  @SuppressWarnings("unchecked")
  public GEN_TYPE create()
  {
    GEN_TYPE returnObj = null;
    try
    {
      returnObj = (GEN_TYPE) getClassType().newInstance();
    }
    catch (InstantiationException e)
    {
      e.printStackTrace();
    }
    catch (IllegalAccessException e)
    {
      e.printStackTrace();
    }
    return returnObj;
  }

  @SuppressWarnings("unchecked")
  public List<GEN_TYPE> loadCompleteList()
  {
    Transaction trans = null;
    Session session = getSession();
    trans = beginTransaction(session);
    List<GEN_TYPE> returnObj = null;
    try
    {
      Query q = session.createQuery("from " + getClassTable() + " as ev");
      session.flush();
      returnObj = q.list();
      trans.rollback();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      trans.rollback();
    }
    return returnObj;

  }

  protected Transaction beginTransaction(Session session)
  {
    Transaction trans = null;
    try
    {
      trans = session.beginTransaction();
    }
    catch (RuntimeException e)
    {
      e.printStackTrace();
      throw e;
    }
    return trans;
  }

  public void save(GEN_TYPE objToSave)
  {
    Session session = getSession();
    Transaction trans = beginTransaction(session);
    try
    {
      session.saveOrUpdate(objToSave);
      trans.commit();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      trans.rollback();
    }
  }

  public void deleteList(List<GEN_TYPE> list)
  {
    Session session = getSession();
    Transaction trans = beginTransaction(session);
    try
    {
      for (GEN_TYPE item : list)
      {
        session.delete(item);
      }
      trans.commit();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      trans.rollback();
    }
  }

  public void deleteAll()
  {
    deleteList(loadCompleteList());
  }

  public void delete(GEN_TYPE item)
  {
    Session session = getSession();
    Transaction trans = beginTransaction(session);
    try
    {
      session.delete(item);
      trans.commit();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      trans.rollback();
    }
  }

  protected boolean execute(HibernateTransactionHelper helper)
  {
    Session session = getSession();
    Transaction trans = session.beginTransaction();
    try
    {
      helper.run(session, trans);
      trans.commit();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      System.err.println("calling transaction.rollback() ... ");
      trans.rollback();
    }
    return false;
  }

  protected Session getSession()
  {
    if (myHibernateDaoFactory == null)
    {
      throw new IllegalArgumentException(
          "It seems like HibernateDao::setHibernateDaoFactory had not been called.");
    }
    return myHibernateDaoFactory.getSession();
  }

  public HibernateDaoFactory getHibernateDaoFactory()
  {
    return myHibernateDaoFactory;
  }

}
