package com.seitenbau.testing.shared.dao.hibernate;

import java.util.Properties;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

import com.seitenbau.testing.shared.config.RepositoryConfig;
import com.seitenbau.testing.shared.dao.IDaoFactory;
import com.seitenbau.testing.shared.dao.IInvocationBaseDao;
import com.seitenbau.testing.shared.dao.IInvocationMethodDao;
import com.seitenbau.testing.shared.dao.IInvocationNewOperatorDao;
import com.seitenbau.testing.shared.dao.IObjectSnapshotDao;
import com.seitenbau.testing.shared.dao.IRecordConfigDao;
import com.seitenbau.testing.shared.dao.IRecordedTestCaseDao;
import com.seitenbau.testing.shared.dao.IRecordedTestDao;

public class HibernateDaoFactory implements IDaoFactory
{

  private Configuration myConfiguration = null;

  private Properties myProperties = null;

  public void forceClear()
  {
    myConfiguration = null;
  }

  public Configuration getConfiguration()
  {
    if (myConfiguration == null)
    {
      myConfiguration = new Configuration();
      myConfiguration.configure();
      if (myProperties != null)
      {
        myConfiguration.setProperties(myProperties);
      }
    }
    return myConfiguration;
  }

  public IRecordedTestDao getRecordedTestDao()
  {
    return (IRecordedTestDao) setupDaoFactory(new RecordedTestDao());
  }

  public IObjectSnapshotDao getObjectSnapshotDao()
  {
    return (IObjectSnapshotDao) setupDaoFactory(new ObjectSnapshotDao());
  }

  public IRecordConfigDao getRecordConfigDao()
  {
    return (IRecordConfigDao) setupDaoFactory(new RecordConfigDao());
  }

  public IInvocationBaseDao getCallToBaseDao()
  {
    return (IInvocationBaseDao) setupDaoFactory(new InvocationBaseDao());
  }

  public IRecordedTestCaseDao getRecordedTestCaseDao()
  {
    return (IRecordedTestCaseDao) setupDaoFactory(new RecordedTestCaseDao());
  }

  @SuppressWarnings("unchecked")
  private Object setupDaoFactory(Object dao)
  {
    if (dao instanceof HibernateDao)
    {
      ((HibernateDao) dao).setHibernateDaoFactory(this);
    }
    return dao;
  }

  public IInvocationNewOperatorDao getCallToBaseNewOperationDao()
  {
    return (IInvocationNewOperatorDao) setupDaoFactory(new InvocationNewOperatorDao());
  }

  public IInvocationMethodDao getCallToMethodDao()
  {
    return (IInvocationMethodDao) setupDaoFactory(new InvocationMethodDao());
  }

  public void setProperties(Properties props)
  {
    myProperties = props;
    myConfiguration.setProperties(myProperties);
  }

  public Properties getProperties()
  {
    if (myProperties == null)
    {
      myProperties = getConfiguration().getProperties();
    }
    return myProperties;
  }

  public void setConfiguration(RepositoryConfig dbcfg)
  {
    if (dbcfg != null)
    {
      Properties prop = getProperties();
      if (prop == null)
      {
        prop = new Properties();
        setProperties(prop);
      }
      putNotNull(prop, "hibernate.connection.url", dbcfg.getDatabaseURL());
      putNotNull(prop, "hibernate.connection.username", dbcfg.getUsername());
      putNotNull(prop, "hibernate.connection.password", dbcfg.getPassword());
      putNotNull(prop, "hibernate.connection.driver_class", dbcfg
          .getDriverClass());
    }
  }

  private void putNotNull(Properties prop, String key, String value)
  {
    if (prop != null && key != null && value != null)
    {
      prop.put(key, value);
    }
  }

  private SessionFactory mySessionFactory;

  private org.hibernate.classic.Session mySession;

  protected SessionFactory getSessionFactory()
  {
    if (mySessionFactory == null)
    {
      try
      {
        mySessionFactory = getConfiguration().buildSessionFactory();
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
    }
    return mySessionFactory;
  }

  protected Session getSession()
  {
    if (mySession == null)
    {
      mySession = getSessionFactory().openSession();
    }
    return mySession;
  }

  protected void close()
  {
    if (mySession != null)
    {
      mySession.close();
    }
  }

  public void detach()
  {
    if (mySession != null)
    {
      mySession.disconnect();
      mySession = null;
    }
    if (mySessionFactory != null)
    {
      mySessionFactory.close();
      mySessionFactory = null;
    }
  }

}
