package com.seitenbau.testing.shared.dao.hibernate;

import java.io.Serializable;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Expression;

import com.seitenbau.testing.shared.dao.IRecordConfigDao;
import com.seitenbau.testing.shared.model.RecordedMethodConfiguration;

public class RecordConfigDao extends HibernateDao<RecordedMethodConfiguration>
    implements IRecordConfigDao
{

  protected Class<?> getClassType()
  {
    return RecordedMethodConfiguration.class;
  }

  public RecordedMethodConfiguration findOrCreateByID(int id)
  {
    Session session = getSession();
    Transaction trans = beginTransaction(session);
    RecordedMethodConfiguration returnObj = null;
    try
    {
      returnObj = (RecordedMethodConfiguration) session.get(getClassType(), id);
      if (returnObj == null)
      {
        // Create new one
        returnObj = create();
        returnObj.setId(id);
        Serializable s = session.save(returnObj);
        System.out.println("Created new RecordConfig with ID: " + s);
        trans.commit();
      }
      else
      {
        trans.rollback();
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
      trans.rollback();
    }
    return returnObj;
  }

  public RecordedMethodConfiguration findOrCreateBy(String packageName,
      String className, String methodName, String returnType, String parameters)
  {
    Session session = getSession();
    Transaction trans = beginTransaction(session);
    RecordedMethodConfiguration returnObj = null;
    try
    {
      Criteria criteria = session
          .createCriteria(RecordedMethodConfiguration.class);
      criteria.add(Expression.eq("ClassPackage", packageName));
      criteria.add(Expression.eq("ClassName", className));
      criteria.add(Expression.eq("MethodName", methodName));
      criteria.add(Expression.eq("ReturnType", returnType));
      criteria.add(Expression.eq("Parameters", parameters));
      List<?> recordList = criteria.list();
      if (recordList.size() == 1)
      {
        returnObj = (RecordedMethodConfiguration) recordList.get(0);
        trans.rollback();
      }
      else
      {
        // Nothing found ... create new ...
        returnObj = create();
        returnObj.setClassPackage(packageName);
        returnObj.setClassName(className);
        returnObj.setMethodName(methodName);
        returnObj.setReturnType(returnType);
        returnObj.setParameters(parameters);
        session.save(returnObj);
        trans.commit();
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
      trans.rollback();
    }

    return returnObj;

  }

  public RecordedMethodConfiguration findBy(String packageName,
      String className, String methodName, String returnType, String parameters)
  {
    Session session = getSession();
    Transaction trans = beginTransaction(session);
    RecordedMethodConfiguration returnObj = null;
    try
    {
      Criteria criteria = session
          .createCriteria(RecordedMethodConfiguration.class);
      criteria.add(Expression.eq("ClassPackage", packageName));
      criteria.add(Expression.eq("ClassName", className));
      criteria.add(Expression.eq("MethodName", methodName));
      criteria.add(Expression.eq("ReturnType", returnType));
      criteria.add(Expression.eq("Parameters", parameters));
      List<?> recordList = criteria.list();
      if (recordList.size() == 1)
      {
        returnObj = (RecordedMethodConfiguration) recordList.get(0);

      }
      else
      {
        returnObj = null;
      }
      trans.rollback();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      trans.rollback();
    }
    return returnObj;

  }

}
