package com.seitenbau.testing.shared.filters;

import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import com.seitenbau.testing.shared.helper.FileHelper;
import com.seitenbau.testing.shared.helper.Generics;
import com.seitenbau.testing.shared.helper.XslTransformer;
import com.seitenbau.testing.shared.helper.XsltBuilder;
import com.seitenbau.testing.shared.locator.ITestPathClassBasedLocator;
import com.seitenbau.testing.shared.model.ObjectSnapshot;
import com.thoughtworks.xstream.XStream;

public class FilterController
{

  private List<String> myXsltList;

  public FilterController(Class<?> clazz, ITestPathClassBasedLocator locator)
  {
    loadConfigFile(clazz, locator);
  }

  private void loadConfigFile(Class<?> clazz,
      ITestPathClassBasedLocator filePrefix)
  {
    String filename = filePrefix.getTestResourceFile("filter", "filter.xml");
    InputStream stream = clazz.getResourceAsStream(filename);
    if (stream != null)
    {
      Object obj = new XStream().fromXML(stream);
      if (!(obj instanceof List<?>))
      {
        // XXX: Handle invalid file content
      }
      else
      {
        List<String> files = Generics.cast(obj);
        for (String file : files)
        {
          loadConfigXslt(clazz, filePrefix, file);
        }
      }
    }
  }

  private void loadConfigXslt(Class<?> clazz,
      ITestPathClassBasedLocator locator, String file)
  {
    String filename = locator.getTestResourceFile("filter", file);
    // XXX: sollte eigentlich ber den Stream gelesen werden
    // InputStream stream = clazz.getResourceAsStream(filename);
    filename = filename.replace('\\', '/');
    URL urlFile = clazz.getResource(filename);
    String filename2 = urlFile.getFile();
    filename2 = filename2.substring(1);

    String content = FileHelper.readFile(filename2);

    getXSLTList().add(content);
  }

  private List<String> getXSLTList()
  {
    if (myXsltList == null)
    {
      myXsltList = new ArrayList<String>();
    }
    return myXsltList;
  }

  public String toXml(Object param1)
  {
    XStream xs = getXStream();
    return xs.toXML(param1);
  }

  private XStream getXStream()
  {
    return new XStream();
  }

  public String filterXml(String xml, String xslt)
  {
    XslTransformer xt = null;

    xt = new XslTransformer();

    String newXml = xt.transform(xslt, xml);
    return newXml;
  }

  public static String buildXslt(String... excludeNodes)
  {
    XsltBuilder xb = new XsltBuilder();
    xb.beginTemplateMatch("node()|@*").beginCopy().applyTemplate("node()|@*")
        .endCopy().endTemplateMatch();
    for (String node : excludeNodes)
    {
      xb.beginTemplateMatch(node).endTemplateMatch();
    }
    return xb.getXSLT();
  }

  public String filterXml(String param1XML)
  {
    String xml = param1XML;
    for (String filterXslt : getXSLTList())
    {
      xml = filterXml(xml, filterXslt);
    }
    return xml;
  }

  public String getFilteredXML(ObjectSnapshot objSnapshot)
  {
    String xml = objSnapshot.getXMLString();
    return filterXml(xml);
  }

}
