package com.seitenbau.testing.shared.helper;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import com.seitenbau.testing.shared.config.ExcludeFieldConfiguration;
import com.seitenbau.testing.shared.model.InvocationBase;
import com.seitenbau.testing.shared.model.InvocationMethod;
import com.seitenbau.testing.shared.model.InvocationNewOperator;
import com.seitenbau.testing.shared.model.ListenerData;
import com.seitenbau.testing.shared.model.ObjectSnapshot;
import com.seitenbau.testing.shared.model.RecordedMethodConfiguration;
import com.seitenbau.testing.shared.model.RecordedTest;
import com.seitenbau.testing.shared.model.RecordedTestCase;


/**
 * Hilfe eine tiefe Kopie zuziehen.
 *
 * Dabei werden Hibernate Proxyies entfertn.
 */
public class DeepCopy
{

  public static RecordedMethodConfiguration createCopy(
      RecordedMethodConfiguration config)
  {
    RecordedMethodConfiguration newConfig = new RecordedMethodConfiguration();
    clone(config, newConfig);

    List<RecordedTestCase> list = new Vector<RecordedTestCase>();
    for (RecordedTestCase tCase : config.getRecordedTestCases())
    {
      RecordedTestCase cpy = createCopy(tCase);
      list.add(cpy);
      cpy.setRecordedMethodConfiguration(newConfig);
    }
    newConfig.setRecordedTestCases(list);

    List<ExcludeFieldConfiguration> fields = new Vector<ExcludeFieldConfiguration>();
    for (ExcludeFieldConfiguration tCase : config.getExcludeFields())
    {
      ExcludeFieldConfiguration cpy = createCopy(tCase);
      fields.add(cpy);
    }
    newConfig.setExcludeFields(fields);

    return newConfig;
  }

  private static ExcludeFieldConfiguration createCopy(
      ExcludeFieldConfiguration orginal)
  {
    ExcludeFieldConfiguration eConfig = new ExcludeFieldConfiguration();
    eConfig.setClassName(orginal.getClassName());
    eConfig.setFieldName(orginal.getFieldName());
    return eConfig;
  }

  private static void clone(RecordedMethodConfiguration config,
      RecordedMethodConfiguration newConfig)
  {
    newConfig.setClassName(config.getClassName());
    newConfig.setClassPackage(config.getClassPackage());

    newConfig.setMethodName(config.getMethodName());
    newConfig.setName(config.getName());
    newConfig.setParameters(config.getParameters());
    newConfig.setProjectName(config.getProjectName());
    newConfig.setReturnType(config.getReturnType());

  }

  public static RecordedTestCase createCopy(RecordedTestCase rTestCase)
  {
    RecordedTestCase newCase = new RecordedTestCase();

    clone(rTestCase, newCase);

    List<RecordedTest> list = new Vector<RecordedTest>();

    for (RecordedTest orginalOccation : rTestCase.getRecordedTests())
    {
      list.add(createCopy(orginalOccation));
    }

    newCase.setRecordedTests(list);
    newCase.setRecordedMethodConfiguration(null);

    return newCase;
  }

  public static RecordedTest createCopy(RecordedTest orginalOccation)
  {
    RecordedTest newOccation = new RecordedTest();
    copyCallToBase(orginalOccation, newOccation);

    newOccation.setMethodName(orginalOccation.getMethodName());
    newOccation.setTrace(createCopy(orginalOccation.getTrace()));

    newOccation.setExceptionThrown(orginalOccation.isExceptionThrown());
    newOccation.setExceptionType(orginalOccation.getExceptionType());
    newOccation.setExceptionValue(createCopy(orginalOccation
        .getExceptionValue()));
    newOccation.setVoidMethod(orginalOccation.isVoidMethod());
    newOccation.setData(createCopy(orginalOccation.getData()));
    newOccation.setModifiers( orginalOccation.getModifiers() );


    return newOccation;
  }

  // private static Map<String, ListenerData> createCopy(
  // Map<String, ListenerData> data) {
  // return null;
  // }

  private static Map<String, ListenerData> createCopy(
      Map<String, ListenerData> oldMap)
  {
    HashMap<String, ListenerData> newMap = new HashMap<String, ListenerData>();
    for (Iterator<String> it = oldMap.keySet().iterator(); it.hasNext();)
    {
      String key = it.next();
      ListenerData old = oldMap.get(key);
      newMap.put(key, createCopy(old));
    }
    return newMap;
  }

  private static ListenerData createCopy(ListenerData old)
  {
    ListenerData ld = new ListenerData();
    ld.setBinayData(old.getBinayData());
    return ld;
  }

  private static void copyCallToBase(InvocationBase orginal, InvocationBase copy)
  {
    copy.setId(orginal.getId());
    copy.setArgumentTypes(orginal.getArgumentTypes());
    copy.setArgumentValues(orginal.getArgumentValues());
    copy.setClassName(orginal.getClassName());
    copy.setPostState(createCopy(orginal.getPostState()));
    copy.setPreState(createCopy(orginal.getPreState()));
    copy.setExceptionType(orginal.getExceptionType());
    copy.setExceptionValue(createCopy(orginal.getExceptionValue()));
    copy.setExceptionThrown(orginal.isExceptionThrown());
    copy.setReturnType(orginal.getReturnType());
    copy.setReturnValue(createCopy(orginal.getReturnValue()));
    copy.setData(createCopy(orginal.getData()));

  }

  private static List<InvocationBase> createCopy(List<InvocationBase> trace)
  {
    Vector<InvocationBase> newList = new Vector<InvocationBase>();
    for (InvocationBase liveCall : trace)
    {
      InvocationBase newCallToBase;
      if (liveCall instanceof InvocationMethod)
      {
        InvocationMethod lcm = new InvocationMethod();
        lcm.setMethodName(((InvocationMethod) liveCall).getMethodName());
        lcm.setModifiers(((InvocationMethod) liveCall).getModifiers());
        newCallToBase = lcm;
      }
      else if (liveCall instanceof InvocationNewOperator)
      {
        newCallToBase = new InvocationNewOperator();
      }
      else
      {
        throw new RuntimeException("Sorry, unsuported CallToBase subclass");
      }
      newCallToBase.setExceptionType(liveCall.getExceptionType());
      newCallToBase.setExceptionValue(createCopy(liveCall.getExceptionValue()));
      newCallToBase.setExceptionThrown(liveCall.isExceptionThrown());
      newCallToBase.setArgumentTypes(liveCall.getArgumentTypes());
      newCallToBase.setArgumentValues(liveCall.getArgumentValues());
      newCallToBase.setClassName(liveCall.getClassName());
      newCallToBase.setId(liveCall.getId());
      newCallToBase.setPostState(createCopy(liveCall.getPostState()));
      newCallToBase.setPreState(createCopy(liveCall.getPreState()));
      newCallToBase.setReturnType(liveCall.getReturnType());
      newCallToBase.setReturnValue(createCopy(liveCall.getReturnValue()));

      newList.add(newCallToBase);
    }
    return newList;
  }

  private static ObjectSnapshot createCopy(ObjectSnapshot orginalObjectSnaphot)
  {
    if (orginalObjectSnaphot == null)
    {
      return null;
    }

    ObjectSnapshot newObjectSnapshot = new ObjectSnapshot();
    newObjectSnapshot.setId(orginalObjectSnaphot.getId());
    newObjectSnapshot.setXMLString(orginalObjectSnaphot.getXMLString());
    return newObjectSnapshot;
  }

  private static void clone(RecordedTestCase orginal, RecordedTestCase clone)
  {
    clone.setDescription(orginal.getDescription());
    clone.setName(orginal.getName());
    clone.setId(orginal.getId());
    clone.setTraceRecorded(orginal.getTraceRecorded());
    clone.setData(createCopy(orginal.getData()));
  }

}
