package com.seitenbau.testing.shared.helper;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Helper for Java generics, when used with none generic objects
 */
public final class Generics
{

  /** Only static methods */
  private Generics()
  {
  }

  /**
   * Helper to supress converstion warnings from <?> to <TYPE>
   * 
   * These Warnings are serious, so make sure the passed parameter can
   * never contain anything other then a Iterator with TYPE's in it !!
   */
  @SuppressWarnings("unchecked")
  public static <TYPE> Iterator<TYPE> cast(Iterator<?> installedProviders)
  {
    return (Iterator<TYPE>) installedProviders;
  }

  /**
   * Helps to suppresses conversation warnings when casting from
   * &lt;?&gt; to &lt;TYPE&gt;
   * 
   * These Warnings are serious, so make sure the passed parameter can
   * never contain anything other then a List with TYPE's in it !!
   */
  @SuppressWarnings("unchecked")
  public static <TYPE> List<TYPE> cast(List<?> installedProviders)
  {
    return (List<TYPE>) installedProviders;
  }

  /**
   * Convenient method to suppress conversation warnings when casting from
   * &lt;?&gt; to &lt;TYPE&gt;
   */
  @SuppressWarnings("unchecked")
  public static <TYPE> TYPE cast(Object obj)
  {
    return (TYPE) obj;
  }

  /**
   * Returns a Iterator to iterate thru all Keys of the given Map.
   * 
   * Enables to write <code>for</code> loops like this: <br/>
   * <code>for(KeyType key:Generics.MapKeysIterator( theMap );</code>
   * <br>
   * <code>
   *   Hashmap<String,Object> aHashmap = new Hashmap<String,Object>();<br/>
   *   // ...<br/>
   *   for(String key:Generics.MapKeysIterator( aHashmap ) {<br/>
   *       Object theObjectForTheKey=aHashmap.get(key);<br/>
   *       // ...<br/>
   *   </code>
   * @param theMap
   * @return
   */
  public static <KEY, VALUE> MapKeysIterator<KEY, VALUE> interatorMapKeys(
      Map<KEY, VALUE> theMap)
  {
    return new Generics.MapKeysIterator<KEY, VALUE>(theMap);
  }

  static public class MapKeysIterator<KEY, VALUE> implements Iterable<KEY>
  {
    private Map<KEY, VALUE> myMap;

    public MapKeysIterator(Map<KEY, VALUE> theMap)
    {
      myMap = theMap;
    }

    public Iterator<KEY> iterator()
    {
      return myMap.keySet().iterator();
    }

  }
}
