package com.seitenbau.testing.shared.helper;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

public class ParameterBuilder
{
  private String myParameters;

  private StringTokenizer myTokens;

  private String currentToken;

  private int count;

  public ParameterBuilder(String parameters)
  {
    myParameters = parameters;
  }

  public void reset()
  {
    myTokens = new StringTokenizer(myParameters, ",");
    count = 0;
  }

  public boolean next()
  {
    boolean hasMore = myTokens.hasMoreTokens();
    if (hasMore)
    {
      currentToken = myTokens.nextToken();
      count++;
    }
    return hasMore;
  }

  public String getType()
  {
    return currentToken;
  }

  public String getCount()
  {
    return String.valueOf(count);
  }

  public String getSeperation()
  {
    return getSeperation(", ");
  }

  public String getSeperation(String delim)
  {
    if (myTokens.hasMoreElements())
    {
      return delim;
    }
    else
    {
      return "";
    }
  }

  public String buildSignatureNL()
  {
    reset();
    StringBuffer sb = new StringBuffer();
    while (next())
    {
      sb.append(getType());
      sb.append(" ");
      sb.append(getCount("param"));
      sb.append(getCount("Param"));
      sb.append(getSeperation());
    }
    return sb.toString();
  }

  public String buildSignature()
  {
    reset();
    StringBuffer sb = new StringBuffer();
    while (next())
    {
      sb.append(getType());
      sb.append(" ");
      sb.append(getCount("param"));
      sb.append(getSeperation());
    }
    return sb.toString();
  }

  public String buildTypelessSignature()
  {
    reset();
    StringBuffer sb = new StringBuffer();
    while (next())
    {
      sb.append(getCount("param"));
      sb.append(getSeperation());
    }
    return sb.toString();
  }

  public String buildTypelessSignature(String paramPrefix)
  {
    reset();
    StringBuffer sb = new StringBuffer();
    while (next())
    {
      sb.append(paramPrefix);
      sb.append(getCount("Param"));
      sb.append(getSeperation());
    }
    return sb.toString();
  }

  public String getCount(String namePrefix)
  {
    String.format("%02d", count);
    return namePrefix + count;
  }

  public boolean hasParameters()
  {
    return (myParameters != null && myParameters.length() > 0);
  }

  public List<String> getAsList()
  {
    StringTokenizer tokens = new StringTokenizer(myParameters, ",");
    List<String> list = new ArrayList<String>();
    while (tokens.hasMoreTokens())
    {
      list.add(tokens.nextToken());
    }
    return list;
  }

  public Object buildObjectSnapshotSignature()
  {
    reset();
    StringBuffer sb = new StringBuffer();
    while (next())
    {
      sb.append("/*");
      sb.append(getType());
      sb.append(" */");
      sb.append("ObjectSnapshot");
      sb.append(" ");
      sb.append(getCount("param"));
      sb.append(getSeperation());
    }
    return sb.toString();
  }

  public String buildClassSignature()
  {
    reset();
    StringBuffer sb = new StringBuffer();
    while (next())
    {
      sb.append(getType());
      sb.append(".class");
      sb.append(getSeperation());
    }
    return sb.toString();
  }
}
