package com.seitenbau.testing.shared.helper;

public class XsltBuilder
{

  public static final String XSLT_HEAD = "<?xml version='1.0' encoding='UTF-8'?>"
      + "<xsl:stylesheet version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>";

  public static final String XSLT_FOOT = "</xsl:stylesheet>";

  private StringBuffer buffer = new StringBuffer();

  private boolean footWritten = false;

  public XsltBuilder()
  {
    getBuffer().append(XSLT_HEAD);
    footWritten = false;
  }

  public XsltBuilder beginTemplateMatch(String match)
  {
    getBuffer().append("<xsl:template match='");
    getBuffer().append(match);
    getBuffer().append("'>");
    return this;
  }

  public XsltBuilder copyOfAll()
  {
    getBuffer().append("<xsl:copy-of select='node()|@*'/>");
    return this;
  }

  public XsltBuilder beginCopyOf(String select)
  {
    getBuffer().append("<xsl:copy-of select='");
    getBuffer().append(select);
    getBuffer().append("'/>");
    return this;
  }

  public XsltBuilder endTemplateMatch()
  {
    getBuffer().append("</xsl:template>");
    return this;
  }

  public String getXSLT()
  {
    StringBuffer buf = getBuffer();
    if (!footWritten)
    {
      buf.append(XSLT_FOOT);
      footWritten = true;
    }
    return buf.toString();
  }

  protected StringBuffer getBuffer()
  {
    return buffer;
  }

  protected void setBuffer(StringBuffer buffer)
  {
    this.buffer = buffer;
  }

  public XsltBuilder beginCopy()
  {
    getBuffer().append("<xsl:copy>");
    return this;
  }

  public XsltBuilder endCopy()
  {
    getBuffer().append("</xsl:copy>");
    return this;
  }

  public XsltBuilder applyTemplate(String string)
  {
    getBuffer().append("<xsl:apply-templates select=\"" + string + "\" />");
    return this;

  }
}
