package com.seitenbau.testing.shared.junit;

import java.io.IOException;
import java.io.InputStream;

import junit.framework.AssertionFailedError;

import org.custommonkey.xmlunit.XMLAssert;
import org.custommonkey.xmlunit.XMLUnit;
import org.xml.sax.SAXException;

import com.seitenbau.junit.IDataTestProvider;
import com.seitenbau.junit.XmlDataContext;
import com.seitenbau.junit.model.DDTModelRoot;
import com.seitenbau.junit.model.DDTModelTest;
import com.seitenbau.testing.shared.filters.FilterController;
import com.seitenbau.testing.shared.helper.FileHelper;
import com.seitenbau.testing.shared.helper.Generics;
import com.seitenbau.testing.shared.locator.IProvidesTestPathClassLocator;
import com.seitenbau.testing.shared.locator.ITestPathClassBasedLocator;
import com.seitenbau.testing.shared.model.ObjectSnapshot;
import com.seitenbau.testing.shared.model.ObjectSnapshotConverter;

public class SBRecorderContext extends XmlDataContext implements
    IProvidesTestPathClassLocator
{

  private FilterController myFilterController;

  private SBRecorderDataProvider myXmlProvider;

  public SBRecorderContext(IDataTestProvider xmlDataProvider,
      DDTModelRoot root, DDTModelTest testItem)
  {
    super(xmlDataProvider, root, testItem);
    if (xmlDataProvider instanceof SBRecorderDataProvider)
    {
      myXmlProvider = (SBRecorderDataProvider) xmlDataProvider;
    }
  }

  public FilterController getFilterController()
  {
    return myFilterController;
  }

  public void setFilterController(FilterController filterController)
  {
    myFilterController = filterController;
  }

  public <TYPE> TYPE makeObject(ObjectSnapshot napshotobjectsnapshot)
  {
    if(napshotobjectsnapshot==null)
    {
      return null;
    }
    Object obj = getOSC().makeObject(napshotobjectsnapshot);
    return Generics.cast(obj);
  }

  protected ObjectSnapshotConverter getOSC()
  {
    return myXmlProvider.getOSC();
  }

  public ObjectSnapshot makeObjectSnapshot(Object actualReturnValue)
  {
    return getOSC().makeSnapshot(actualReturnValue);

  }

  /**
   * Retrives the XML repesentation from the ObjectSnapshots. Applies
   * the XML Filters on them and then compares them via XMLUnit.
   *
   * @param expected
   * @param actual
   */
  public void applyXmlFiltersAndAssertEquals(ObjectSnapshot expected,
      ObjectSnapshot actual)
  {

    String expectedXML = getFilterController().getFilteredXML(expected);
    String actualXML = getFilterController().getFilteredXML(actual);

    try
    {
      XMLUnit.setIgnoreWhitespace(true);
      XMLAssert.assertXMLEqual(expectedXML, actualXML);
    }
    catch(AssertionFailedError error) {
      System.out.println("===== expected XML =====");
      System.out.println(expectedXML);
      System.out.println("=====  actual XML  =====");
      System.out.println(actualXML);
      throw error;
    }
    catch (SAXException e)
    {
      throw new RuntimeException(e);
    }
    catch (IOException e)
    {
      throw new RuntimeException(e);
    }
  }

  public void applyXmlFiltersAndAssertEquals(
      ObjectSnapshot expectedReturnValueSnapshot, Object actualReturnValue)
  {
    applyXmlFiltersAndAssertEquals(expectedReturnValueSnapshot,
        makeObjectSnapshot(actualReturnValue));
  }

  public ITestPathClassBasedLocator getLocator()
  {
    return myXmlProvider.getLocator();
  }

  public String getFileContent(String plugin, String postfix)
  {

    Integer idx = getTestItem().getId();
    String filename = getLocator().getTestResourceFile(plugin, idx, postfix);
    InputStream stream = getTestClass().getResourceAsStream(filename);
    if (stream == null)
    {
      throw new RuntimeException("Could not find file [" + filename + "] ");
    }
    String content = FileHelper.readStream(stream);
    return content;
  }

  public Object makeObject(String content)
  {
    ObjectSnapshot shot = new ObjectSnapshot();
    shot.setXMLString(content);
    return makeObject(shot);
  }
}
