package com.seitenbau.testing.shared.junit;

import java.util.ArrayList;
import java.util.List;

import com.seitenbau.junit.ITestContext;
import com.seitenbau.junit.TestData;
import com.seitenbau.junit.XmlDataProvider;
import com.seitenbau.junit.model.DDTModelRoot;
import com.seitenbau.junit.model.DDTModelTest;
import com.seitenbau.testing.shared.config.ExcludeFieldConfiguration;
import com.seitenbau.testing.shared.filters.FilterController;
import com.seitenbau.testing.shared.locator.ITestPathClassBasedLocator;
import com.seitenbau.testing.shared.locator.TestPathClassBasedLocator;
import com.seitenbau.testing.shared.model.ObjectSnapshot;
import com.seitenbau.testing.shared.model.ObjectSnapshotConverter;
import com.seitenbau.testing.shared.xstream.XStreamObjectSnapshotSweetener;
import com.thoughtworks.xstream.XStream;

public class SBRecorderDataProvider extends XmlDataProvider
{

  @Override
  protected void setupXStreamFields(XStream xs)
  {
    super.setupXStreamFields(xs);
    xs.alias("ObjectSnapshot", ObjectSnapshot.class);
  }

  private ObjectSnapshotConverter myOSC;
  private DDTModelRoot myModelRoot;

  @Override
  protected void setupXStreamConverters(XStream xs)
  {
    xs.registerConverter(new XStreamObjectSnapshotSweetener());
  }

  @Override
  protected ITestContext createContext(int seqNr, DDTModelRoot model,
      DDTModelTest testItem)
  {
    SBRecorderContext context = new SBRecorderContext(this, model, testItem);
    myModelRoot=model;
    ITestPathClassBasedLocator locator = getLocator();
    FilterController fc = new FilterController(getTestClass(), locator);
    context.setFilterController(fc);
    return context;
  }

  public ITestPathClassBasedLocator getLocator()
  {
    String name = myModelRoot.getTargetClassName();
    String testName = getTestClass().getSimpleName();
    return new TestPathClassBasedLocator( name ,testName);
  }

  @Override
  protected TestData createTestData(int sequenceNumber, DDTModelRoot testfile,
      DDTModelTest testItem)
  {

    return new TestData(this, sequenceNumber, testfile, testItem)
    {

      @Override
      protected void buildRealParameters(List<Object> parameters)
      {
        List<Object> realParameters = getTestItem().getParameters();
        if(realParameters!=null)
        {
          for(Object orgObject:realParameters)
          {
            Object newObject = getOSC().makeObject((ObjectSnapshot) orgObject);
            parameters.add( newObject );
          }
        }
      }

//      @Override
//      protected List<Object> buildParameters()
//      {
//        List<Object> parameters = super.buildParameters();
//        List<Object> newParameters=new ArrayList<Object>();
//        int i = 0;
//        if (getTestItem().getExpected() != null)
//        {
//          newParameters.add(parameters.get(0));
//          i = 1;
//        }
//        for (; i < parameters.size(); i++)
//        {
//          Object orgObject=parameters.get(i);
//          Object newObject = getOSC().makeObject((ObjectSnapshot) orgObject);
//          newParameters.add(newObject);
//        }
//        if(isPassContext()) {
//
//        }
//
//        return newParameters;
//      }

//      @Override
//      public Object[] getParameters()
//      {
//        Object[] snapshotParams = super.getParameters();
//        int i = 0;
//        if (getTestItem().getExpected() != null)
//        {
//          i = 1;
//        }
//        for (; i < snapshotParams.length; i++)
//        {
//          Object object = snapshotParams[i];
//          if (object instanceof ObjectSnapshot)
//          {
//            snapshotParams[i] = getOSC().makeObject((ObjectSnapshot) object);
//          }
//        }
//        return snapshotParams;
//      }
    };
  }

  protected ObjectSnapshotConverter getOSC()
  {
    if (myOSC == null)
    {
      List<ExcludeFieldConfiguration> efields=new ArrayList<ExcludeFieldConfiguration>();
      ExcludeFieldConfiguration ecfg=new ExcludeFieldConfiguration();
      ecfg.setClassName("java.lang.Throwable");
      ecfg.setFieldName("trace");
      efields.add( ecfg );
      myOSC = new ObjectSnapshotConverter(null);
    }
    return myOSC;
  }

}
