package com.seitenbau.testing.shared.locator;

import java.io.File;

public class TestPathFileBasedLocator extends TestPathLocator implements
    ITestPathFileBasedLocator
{
  private String myTestRootFolder;

  private String myTestResourceFolder;

  private String myClassPackage;

  public TestPathFileBasedLocator(String testPrefix ,String testRootFolder,
      String testResourcesRootFolder, String classPackage, String className)
  {
    super(testPrefix,className);
    myTestRootFolder = testRootFolder;
    myTestResourceFolder = testResourcesRootFolder;
    myClassPackage = classPackage;
    myClassName = className;
  }

  public String getTestFolder()
   {
    return getAndMakeTestFolder();
  }
  public String getTestFile(String fileName)
  {
    String testFolder = getAndMakeTestFolder();
    if (fileName == null)
    {
      return testFolder;
    }
    else
    {
      return testFolder + getTestPrefix() + fileName;
    }
  }

  public String getTestResourceFile(String fileName)
  {
    String testResourceFolder = getAndMakeTestResourceFolder();
    if (fileName == null)
    {
      return testResourceFolder;
    }
    else
    {
      return testResourceFolder + getTestPrefix() + fileName;
    }
  }

  public String getTestResourceFile(String pluginName, int runningNumber,
      String fileName)
  {
    String pluginRootFolder = getAndMakePluginResourceFolder(pluginName);

    String pluginFileName = getPluginFileNameOnly(pluginName, runningNumber,
        fileName);

    return pluginRootFolder + pluginFileName;
  }

  public String getTestResourceFile(String pluginName, String fileName)
  {
    String pluginfolder = getAndMakePluginResourceFolder(pluginName);

    String pluginFileName = getPluginFileNameOnly(pluginName, fileName);

    return pluginfolder + pluginFileName;
  }

  private String getAndMakePluginResourceFolder(String pluginName)
  {
    String folderPack = getAndMakeTestResourceFolder();

    String pluginFolder = getPluginFolder(pluginName);

    String folder = asPathString(folderPack + pluginFolder);

    new File(folder).mkdirs();

    return folder;
  }

  private String getAndMakeTestResourceFolder()
  {
    String folder = asPathString(getTestResourceFolder2());

    String folderPack = asPathString(folder + getClassPackageAsFolder());

    new File(folderPack).mkdirs();

    return folderPack;
  }

  /**
   * @return The Path to where the Test Class is generated. This
   *         includes the Test-Root-Source Folder and the Package as
   *         Path. If the path dosn't exist, it will be created. (eg.
   *         ./src/main/test/com/example/dao/)
   */
  private String getAndMakeTestFolder()
  {
    StringBuffer buf = new StringBuffer();

    buf.append(asPathString(getTestRootFolder()));
    buf.append(getClassPackageAsFolder());

    String rootAndPackagePath = asPathString(buf.toString());
    new File(rootAndPackagePath).mkdirs();

    return rootAndPackagePath;
  }

  /**
   * @return The Package as Path, this only replaces '.' with the
   *         System's Path seperator. e.g.( com.example.dao --&gt;
   *         com/example/dao)
   */
  private String getClassPackageAsFolder()
  {
    return getClassPackage().replace('.', File.separatorChar);
  }

  /**
   * @return The Test-Root-Folder (eg. "./src/main/test") or whatever
   *         was set vie setTestRootFolder() or via the Constructor.
   *         If the path was not set/ is <code>null</code> then a
   *         empty "" String will be returned.
   */
  public String getTestRootFolder()
  {
    if (myTestRootFolder == null)
    {
      return "";
    }
    return myTestRootFolder;
  }

  /**
   * @param testRootFolder Set the Test-Root-Folder (e.g.
   *        "./src/main/test")
   */
  public void setTestRootFolder(String testRootFolder)
  {
    myTestRootFolder = testRootFolder;
  }

  /**
   * @return The Test-Root-Resource-Folder (eg.
   *         "./src/main/ressources") or whatever was set vie
   *         setTestResourceFolder() via the Constructor. If the path
   *         was not set / is <code>null</code> then the
   *         Test-Root-Folder will be returend by a subsequent call to
   *         getTestRootFolder();
   */
  public String getTestResourceFolder2()
  {
    if (myTestResourceFolder == null)
    {
      return getTestRootFolder();
    }
    return myTestResourceFolder;
  }

  /**
   * @param testResourceFolder Set the Test-Root-Resource-Folder (e.g.
   *        "./src/main/ressources")
   */
  public void setTestResourceFolder(String testResourceFolder)
  {
    myTestResourceFolder = testResourceFolder;
  }

  /**
   * @return get the package of the TestClass
   */
  public String getClassPackage()
  {
    if (myClassPackage == null)
    {
      return "";
    }
    return myClassPackage;
  }

  /**
   * @param classPackage Set the Test Class Package
   */
  public void setClassPackage(String classPackage)
  {
    myClassPackage = classPackage;
  }

  public String getTestResourceFolder()
  {
    return getAndMakeTestResourceFolder();
  }

}
