package com.seitenbau.testing.shared.model;

/**
 * Wurzel-Modell fr alle Arten von 'Aufrufen', Methoden, Operatoren
 * etc.
 * 
 * 
 */
public abstract class InvocationBase extends DataModel
{
  private ObjectSnapshot myPreState = null;

  private ObjectSnapshot myPostState = null;

  private ObjectSnapshot myReturnValue = null;

  private ObjectSnapshot[] myArgumentValues = null;

  private String myReturnType = null;

  private String[] myArgumentTypes = null;

  private String myClassName = null;

  /** set to true if an exception was thrown */
  private boolean myExceptionThrown = false;

  /** The FQN of the thrown Exception */
  private String myExceptionType = null;

  /** The Value as ObjectSnapshot of the thrown Exception */
  private ObjectSnapshot myExceptionValue = null;

  public ObjectSnapshot[] getArgumentValues()
  {
    return myArgumentValues;
  }

  public void setArgumentValues(ObjectSnapshot[] argumentValues)
  {
    myArgumentValues = argumentValues;
  }

  public ObjectSnapshot getReturnValue()
  {
    return myReturnValue;
  }

  public void setReturnValue(ObjectSnapshot returnValue)
  {
    myReturnValue = returnValue;
  }

  public ObjectSnapshot getPreState()
  {
    return myPreState;
  }

  public void setPreState(ObjectSnapshot preState)
  {
    myPreState = preState;
  }

  public ObjectSnapshot getPostState()
  {
    return myPostState;
  }

  public void setPostState(ObjectSnapshot postState)
  {
    myPostState = postState;
  }

  public String getReturnType()
  {
    return myReturnType;
  }

  public void setReturnType(Class<?> type)
  {
    if (type != null)
    {
      myReturnType = type.getCanonicalName();
    }
    else
    {
      myReturnType = null;
    }
  }

  public void setReturnType(String returnType)
  {
    myReturnType = returnType;
  }

  public String[] getArgumentTypes()
  {
    return myArgumentTypes;
  }

  public void setArgumentTypes(String[] argumentTypes)
  {
    myArgumentTypes = argumentTypes;
  }

  public void setArgumentTypes(Class<?>[] types)
  {
    if (types == null)
    {
      myArgumentTypes = null;
    }
    else if (types.length == 0)
    {
      myArgumentTypes = null;
    }
    else
    {
      String[] ta = new String[types.length];
      for (int i = 0; i < types.length; ++i)
      {
        Class<?> c = types[i];
        ta[i] = c.getCanonicalName();
      }
      myArgumentTypes = ta;
    }
  }

  public String getClassName()
  {
    return myClassName;
  }

  public void setClassName(String clazzName)
  {
    myClassName = clazzName;
  }

  public String getExceptionType()
  {
    return myExceptionType;
  }

  public void setExceptionType(String exceptionType)
  {
    myExceptionType = exceptionType;
  }

  public ObjectSnapshot getExceptionValue()
  {
    return myExceptionValue;
  }

  public void setExceptionValue(ObjectSnapshot exceptionValue)
  {
    myExceptionValue = exceptionValue;
  }

  public boolean isExceptionThrown()
  {
    return myExceptionThrown;
  }

  public void setExceptionThrown(boolean exceptionThrown)
  {
    myExceptionThrown = exceptionThrown;
  }
}
