package com.seitenbau.testing.shared.tracer;

import java.util.LinkedList;
import java.util.List;

import com.seitenbau.testing.shared.model.RecordedTestCase;


/**
 * Implementiert die TracerListener Funktionalitt.
 *
 */
public abstract class AbstractTracerWithListener extends
    AbstractTracerWithHelpers implements ITracerWithListener
{

  private List<ITracerListener> myTracerListeners = null;

  /**
   * Get List of all curren registerd Listeners
   *
   * @return The List of Listeners.
   */
  public List<ITracerListener> getListeners()
  {
    if (myTracerListeners == null)
    {
      myTracerListeners = new LinkedList<ITracerListener>();
    }
    return myTracerListeners;
  }

  /**
   * @see ITracerWithListener#addTracerListener(ITracerListener)
   */
  public void addTracerListener(ITracerListener theListener)
  {
    getListeners().add(theListener);
  }

  /**
   * @see ITracerWithListener#removeTracerListener(ITracerListener)
   */
  public void removeTracerListener(ITracerListener theListener)
  {
    getListeners().remove(theListener);
  }

  @Override
  protected void finishUp()
  {
    try
    {
      fireFinished();
      super.finishUp();
    }
    catch (Throwable throwed)
    {
      System.err
          .println("===== Catched an internal Tracer Listener Error! =====");
      throwed.printStackTrace();
    }
  }

  /**
   * Informiert alle angemeldeten Listeners ber das Betreten der
   * Methode
   *
   * @param recordedTest Aktuelles RecordedTest Object
   */
  protected void fireEnteringMethod()
  {
    try
    {
      for (ITracerListener listener : getListeners())
      {
        if (listener != null)
        {
          listener.enteringTargetMethod(getRecordedTestInstance());
        }
      }
    }
    catch (Throwable throwed)
    {
      System.err
          .println("===== Catched an internal Tracer Listener Error! =====");
      throwed.printStackTrace();
    }
  }

  protected void fireSubMethodEntering()
  {
    try
    {
      for (ITracerListener listener : getListeners())
      {
        if (listener != null)
        {
          listener.subMethodEntering(getSubMethodInstance());
        }
      }
    }
    catch (Throwable throwed)
    {
      System.err.println("===== Catched an internal Tracer Listener Error! =====");
      throwed.printStackTrace();
    }
  }

  protected void fireSubMethodLeaving(Object theReturnValue)
  {
    try
    {
      for (ITracerListener listener : getListeners())
      {
        if (listener != null)
        {
          listener.subMethodLeaving(getSubMethodInstance(), theReturnValue);
        }
      }
    }
    catch (Throwable throwed)
    {
      System.err
          .println("===== Catched an internal Tracer Listener Error! =====");
      throwed.printStackTrace();
    }
  }

  protected void fireNewOperatorLeaving(Object theReturnValue)
  {
    try
    {
      for (ITracerListener listener : getListeners())
      {
        if (listener != null)
        {
          listener.newOperatorLeaving(getNewOperatorInstance(), theReturnValue);
        }
      }
    }
    catch (Throwable throwed)
    {
      System.err
          .println("===== Catched an internal Tracer Listener Error! =====");
      throwed.printStackTrace();
    }
  }

  protected void fireNewOperatorEntering()
  {
    try
    {
      for (ITracerListener listener : getListeners())
      {
        if (listener != null)
        {
          listener.newOperatorEntering(getNewOperatorInstance());
        }
      }
    }
    catch (Throwable throwed)
    {
      System.err
          .println("===== Catched an internal Tracer Listener Error! =====");
      throwed.printStackTrace();
    }
  }

  /**
   * Informiert alle angemeldeten Listeners ber das Verlassen der
   * Methode
   *
   * @param recordedTest Aktuelles RecordedTest Object
   */
  protected void fireLeavingMethod(Object theReturnValue)
  {
    try
    {
      for (ITracerListener listener : getListeners())
      {
        if (listener != null)
        {
          listener.leavingTargetMethod(getRecordedTestInstance(),
              theReturnValue);
        }
      }
    }
    catch (Throwable throwed)
    {
      System.err
          .println("===== Catched an internal Tracer Listener Error! =====");
      throwed.printStackTrace();
    }
  }

  protected void fireInitTestCase(RecordedTestCase theCase)
  {
    try
    {
      for (ITracerListener listener : getListeners())
      {
        if (listener != null)
        {
          listener.initTestCase(theCase);
        }
      }
    }
    catch (Throwable throwed)
    {
      System.err
          .println("===== Catched an internal Tracer Listener Error! =====");
      throwed.printStackTrace();
    }
  }

  protected void fireNewOperatorThrowedException(Throwable t)
  {
    try
    {
      for (ITracerListener listener : getListeners())
      {
        if (listener != null)
        {
          listener.newOperatorThrowedException(getNewOperatorInstance(), t);
        }
      }
    }
    catch (Throwable throwed)
    {
      System.err
          .println("===== Catched an internal Tracer Listener Error! =====");
      throwed.printStackTrace();
    }
  }

  protected void fireSubMethodThrowedException(Throwable theThrowable)
  {
    try
    {
      for (ITracerListener listener : getListeners())
      {
        if (listener != null)
        {
          listener.subMethodThrowedException(getSubMethodInstance(),
              theThrowable);
        }
      }
    }
    catch (Throwable throwed)
    {
      System.err
          .println("===== Catched an internal Tracer Listener Error! =====");
      throwed.printStackTrace();
    }
  }

  protected void fireThrowedException(Throwable theThrowable)
  {
    for (ITracerListener listener : getListeners())
    {
      if (listener != null)
      {
        listener.throwedException(getRecordedTestInstance(), theThrowable);
      }
    }
  }

  protected void fireFinished()
  {
    for (ITracerListener listener : getListeners())
    {
      if (listener != null)
      {
        listener.finished(getRecordedTestInstance());
      }
    }
  }

}
