package com.seitenbau.testing.shared.tracer.listener;

import static com.seitenbau.testing.shared.helper.LoggerHelper.logger;

import java.util.List;
import java.util.Vector;

import com.seitenbau.testing.shared.model.InvocationMethod;
import com.seitenbau.testing.shared.model.InvocationNewOperator;
import com.seitenbau.testing.shared.model.RecordedTest;


/**
 * Einfacher Listern welcher alle Benachrichtigungen auf die Konsole
 * ausgibt.
 */
public class ConsoleListener extends DefaultListener
{

  List<String> meineDaten = new Vector<String>();

  public void enteringTargetMethod(RecordedTest theRecordedTest)
  {

    meineDaten.add("enteringTargetMethod");
    registerToStore("ConsoleListener::key:store", meineDaten);

    logger.info("ConsoleListener::enteringMethod(" + theRecordedTest + ");");
  }

  public void leavingTargetMethod(RecordedTest recordedTest,
      Object theReturnValue)
  {
    logger.info("ConsoleListener::leavingTargetMethod( " + recordedTest + ", "
        + theReturnValue + ");");
    meineDaten.add("leavingTargetMethod");

  }

  public void newOperatorEntering(InvocationNewOperator newOperatorInstance)
  {
    logger.info("ConsoleListener::newOperatorEntering(" + newOperatorInstance
        + ");");
    meineDaten.add("newOperatorEntering");
  }

  public void newOperatorLeaving(InvocationNewOperator newOperatorInstance,
      Object theReturnValue)
  {
    logger.info("ConsoleListener::newOperatorLeaving(" + newOperatorInstance
        + ", " + theReturnValue + ");");
    meineDaten.add("newOperatorLeaving");
  }

  public void newOperatorThrowedException(InvocationNewOperator newOperator,
      Throwable theThrowable)
  {
    logger.info("ConsoleListener::newOperatorLeaving(" + newOperator + ", "
        + theThrowable + ");");
    meineDaten.add("newOperatorThrowedException");
  }

  public void subMethodEntering(InvocationMethod subMethodInstance)
  {
    if (subMethodInstance != null)
    {
      logger.info("ConsoleListener::subMethodEntering("
          + subMethodInstance.getMethodName() + "[" + subMethodInstance.getId()
          + "]);");
    }
    else
    {
      logger.info("ConsoleListener::subMethodEntering(null);");
    }
    meineDaten.add("subMethodEntering");
  }

  public void subMethodLeaving(InvocationMethod subMethodInstance,
      Object theReturnValue)
  {
    logger.info("ConsoleListener::subMethodLeaving(" + subMethodInstance + ", "
        + theReturnValue + ");");
    meineDaten.add("subMethodLeaving");
  }

  public void subMethodThrowedException(InvocationMethod subMethodInstance,
      Throwable theThrowable)
  {
    logger.info("ConsoleListener::subMethodThrowedException("
        + subMethodInstance + ", " + theThrowable + ");");
    meineDaten.add("subMethodThrowedException");
  }

  public void throwedException(RecordedTest recordedTestInstance,
      Throwable theThrowable)
  {
    logger.info("ConsoleListener::throwedException(" + recordedTestInstance
        + ", " + theThrowable + ");");
    meineDaten.add("throwedException");
  }

}
