package com.seitenbau.testing.shared.tracer.listener;

import java.util.HashMap;

import com.seitenbau.testing.shared.config.IListenerConfiguration;
import com.seitenbau.testing.shared.config.ListenerConfiguration;
import com.seitenbau.testing.shared.config.MethodConfiguration;
import com.seitenbau.testing.shared.dao.IDaoFactory;
import com.seitenbau.testing.shared.model.InvocationMethod;
import com.seitenbau.testing.shared.model.InvocationNewOperator;
import com.seitenbau.testing.shared.model.ListenerData;
import com.seitenbau.testing.shared.model.RecordedTest;
import com.seitenbau.testing.shared.model.RecordedTestCase;
import com.seitenbau.testing.shared.tracer.ITracerListener;
import com.thoughtworks.xstream.XStream;

/**
 * Minimaler Listener: macht einfach nichts
 */
public abstract class DefaultListener implements ITracerListener
{

  private IDaoFactory myDaoFactory;

  private MethodConfiguration myMethodConfiguration;

  private IListenerConfiguration myListenerConfiguration;

  public void initListener(MethodConfiguration methodConfiguration,
      IListenerConfiguration listenerConfiguration, IDaoFactory daoFactory)
  {
    if (methodConfiguration == null)
    {
      throw new IllegalArgumentException("MethodConfiguration was null");
    }
    if (listenerConfiguration == null)
    {
      throw new IllegalArgumentException("ListenerConfiguration was null");
    }
    myDaoFactory = daoFactory;
    myMethodConfiguration = methodConfiguration;
    if (listenerConfiguration instanceof IListenerConfiguration)
    {
      myListenerConfiguration = (IListenerConfiguration) listenerConfiguration;
      setup(myListenerConfiguration);
    }

  }

  public void initTestCase(RecordedTestCase theCase)
  {
    if (theCase != null)
    {
      registerOnTestCase(theCase);
      getDaoFactory().getRecordedTestCaseDao().save(theCase);
    }
  }

  protected void registerOnTestCase(RecordedTestCase theCase)
  {
  }

  protected void setup(IListenerConfiguration listenerConfiguration)
  {

  }

  public IDaoFactory getDaoFactory()
  {
    return myDaoFactory;
  }

  public void setDaoFactory(IDaoFactory daoFactory)
  {
    myDaoFactory = daoFactory;
  }

  public void enteringTargetMethod(RecordedTest recordedTest)
  {
  }

  public void leavingTargetMethod(RecordedTest recordedTest,
      Object theReturnValue)
  {
  }

  public void newOperatorEntering(InvocationNewOperator newOperatorInstance)
  {
  }

  public void newOperatorLeaving(InvocationNewOperator newOperatorInstance,
      Object theReturnValue)
  {
  }

  public void newOperatorThrowedException(InvocationNewOperator newOperator,
      Throwable theThrowable)
  {
  }

  public void subMethodEntering(InvocationMethod subMethodInstance)
  {
  }

  public void subMethodLeaving(InvocationMethod subMethodInstance,
      Object theReturnValue)
  {
  }

  public void subMethodThrowedException(InvocationMethod subMethodInstance,
      Throwable theThrowable)
  {
  }

  public void throwedException(RecordedTest recordedTestInstance,
      Throwable theThrowable)
  {
  }

  public MethodConfiguration getMethodConfiguration()
  {
    return myMethodConfiguration;
  }

  public void setMethodConfiguration(MethodConfiguration methodConfiguration)
  {
    myMethodConfiguration = methodConfiguration;
  }

  public IListenerConfiguration getListenerConfiguration()
  {
    return myListenerConfiguration;
  }

  public void setListenerConfiguration(
      ListenerConfiguration listenerConfiguration)
  {
    myListenerConfiguration = listenerConfiguration;
  }

  public void finished(RecordedTest recordedTest)
  {
    HashMap<String, Object> store = getStorage();
    for (String key : store.keySet())
    {
      Object objToStore = store.get(key);
      String xml = new XStream().toXML(objToStore);
      ListenerData data = new ListenerData(xml);
      recordedTest.setData(key, data);
    }
  }

  // temp {
  private HashMap<String, Object> myStorage;

  protected void registerToStore(String key, Object toStore)
  {
    getStorage().put(key, toStore);
  }

  private HashMap<String, Object> getStorage()
  {
    if (myStorage == null)
    {
      myStorage = new HashMap<String, Object>();
    }
    return myStorage;
  }
  // temp }

}
