package com.seitenbau.testing.shared.xstream;

import java.io.Writer;

import com.thoughtworks.xstream.io.xml.PrettyPrintWriter;
import com.thoughtworks.xstream.io.xml.XmlFriendlyReplacer;

/**
 * Verhindert das Auftauchen von Control Characters im XML Strom.
 *
 * Also Sonderzeichen <32 werden mit einem \\{} versehen. Bsp: 0 =
 * \{0}
 */
public class HideControlCharPrettyPrintWriter extends PrettyPrintWriter
{

  private static final int ASCII_SPACE_CHAR = 0x20;

  public HideControlCharPrettyPrintWriter(Writer writer,
      XmlFriendlyReplacer replacer)
  {
    super(writer, replacer);
  }

  @Override
  public void setValue(String text)
  {
    char[] cbuf = text.toCharArray();
    StringBuffer out = new StringBuffer();
    for (int i = 0; i < cbuf.length; ++i)
    {
      char c = cbuf[i];
      if (c < ASCII_SPACE_CHAR)
      {
        out.append(encodeChar(c));
      }
      else if (c == '\\')
      {
        out.append("\\\\");
      }
      else
      {
        out.append(c);
      }
    }
    super.setValue(out.toString());

  }

  protected StringBuffer encodeChar(char c)
  {
    StringBuffer sb = new StringBuffer();
    sb.append("\\{");
    sb.append((int) c);
    sb.append("}");
    return sb;
  }

}
