package com.seitenbau.testing.shared.xstream;

import java.io.StringReader;
import java.io.StringWriter;

import com.seitenbau.testing.shared.model.ObjectSnapshot;
import com.thoughtworks.xstream.converters.Converter;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;
import com.thoughtworks.xstream.io.copy.HierarchicalStreamCopier;
import com.thoughtworks.xstream.io.xml.CompactWriter;
import com.thoughtworks.xstream.io.xml.XppReader;

/**
 * ObjectSnapshotConverter
 * 
 * XStream Konverter um die XML Daten aus ObjectSnapshot direkt in
 * eine Ausgabe von XStream einzubetten.
 * 
 * In ObjectSnapshot liegen Objekte als XML Serialisiert als String.
 * Wrde ObjectSnapshot ganz normal von XStream behandelt, wrde
 * dieser String als Value eingefgt. Dadurch geht die Lesbarkeit
 * verloren, da die &gt; und &lt& Zeichen konvertiert werden.
 * 
 * Dieser Konverter lst dieses Probelm indem der Inhalt des String
 * direkt 1:1 in dem Ausgabestrom von XStream eingebettet wird.
 * 
 */
public class XStreamObjectSnapshotSweetener implements Converter
{

  private HierarchicalStreamCopier copier = new HierarchicalStreamCopier();

  @SuppressWarnings("unchecked")
  public boolean canConvert(Class clazz)
  {
    return clazz.equals(ObjectSnapshot.class);
  }

  public void marshal(Object value, HierarchicalStreamWriter writer,
      MarshallingContext context)
  {
    ObjectSnapshot stringBuilder = (ObjectSnapshot) value;

    HierarchicalStreamReader sourceReader = new XppReader(new StringReader(
        stringBuilder.getXMLString()));

    copier.copy(sourceReader, writer);
  }

  public Object unmarshal(HierarchicalStreamReader reader,
      UnmarshallingContext context)
  {
    // Ignoriere die myXXX Tags
    reader.moveDown();

    ObjectSnapshot os = new ObjectSnapshot();
    StringWriter buffer = new StringWriter();
    copier.copy(reader, new CompactWriter(buffer));
    os.setXMLString(buffer.toString());

    // Ignorieren der myXXX Tags wieder aufheben
    reader.moveUp();
    return os;
  }

}
