package com.seitenbau.testing.templates;

import com.seitenbau.testing.shared.templateengine.ITemplateParams;
import com.seitenbau.testing.shared.templateengine.IGenerator;

import com.seitenbau.testing.shared.helper.*;
import com.seitenbau.testing.shared.templateengine.*;
import java.util.*;
import com.seitenbau.testing.shared.model.*;
import com.seitenbau.testing.shared.junit.*;
import com.seitenbau.testing.testgenerator.*;

public class JUnit4_DataDrivenTestInvocationGenerator implements IGenerator
{
  protected static String nl;
  public static synchronized JUnit4_DataDrivenTestInvocationGenerator create(String lineSeparator)
  {
    nl = lineSeparator;
    JUnit4_DataDrivenTestInvocationGenerator result = new JUnit4_DataDrivenTestInvocationGenerator();
    nl = null;
    return result;
  }

  public final String NL = nl == null ? (System.getProperties().getProperty("line.separator")) : nl;
  protected final String TEXT_1 = "    Throwable ";
  protected final String TEXT_2 = "=null;";
  protected final String TEXT_3 = NL + "    ";
  protected final String TEXT_4 = NL + "    try {" + NL + "    \t";
  protected final String TEXT_5 = NL + "    } catch(Throwable theThrowable) {" + NL + "\t\t";
  protected final String TEXT_6 = "=theThrowable;" + NL + "\t\tassertEquals(true, ";
  protected final String TEXT_7 = ".getTestItem().getExpected().isException());" + NL + "" + NL + "\t\tObjectSnapshot expectedException=(ObjectSnapshot) ";
  protected final String TEXT_8 = ".getTestItem().getExpected().getExpectedException();" + NL + "\t\t";
  protected final String TEXT_9 = ".applyXmlFiltersAndAssertEquals(" + NL + "\t\t          expectedException," + NL + "\t\t          actualException);" + NL + "    }";
  protected final String TEXT_10 = NL + "\t";
  protected final String TEXT_11 = NL + "\t";

	/* (non-javadoc)
    * @see IGenerator#generate(Object)
    */
	@SuppressWarnings("unchecked")
	public String generate(ITemplateParams argument)
  {
    final StringBuffer stringBuffer = new StringBuffer();
    
  SimpleModelParam model=(SimpleModelParam)argument;
  RecordedMethodConfiguration method=(RecordedMethodConfiguration)model.getModel();

  boolean catchExceptions=false;
  boolean isStatic=false;
  boolean isPrivate=false;
  boolean isVoid=method.getReturnType().equals("void");
  for(Iterator<RecordedTestCase> testCaseIT=method.getRecordedTestCases().iterator();testCaseIT.hasNext();) {
    RecordedTestCase testCase=testCaseIT.next();
    for(Iterator<RecordedTest> testIT=testCase.getRecordedTests().iterator();testIT.hasNext();) {
      RecordedTest test=testIT.next();
      if( test.isExceptionThrown() ) {
        catchExceptions=true;
      }
      if( test.isVoidMethod() ) {
        isVoid=true;
      }
      if( java.lang.reflect.Modifier.isPrivate(test.getModifiers()) ) {
        isPrivate=true;
      }
      if( java.lang.reflect.Modifier.isStatic(test.getModifiers()) ) {
        isStatic=true;
      }
    }
  }
  ParameterBuilder pb=new ParameterBuilder( method.getParameters() );

  // ************************************************************
  // *    setup some text parts
  // ************************************************************
  StringBuffer returnValueAssignment	= new StringBuffer();
  StringBuffer returnValueDeclaration	= new StringBuffer();

  if( isVoid ) {
    // Nothing to do, keep them empty
  } else {
	returnValueDeclaration	.append( method.getReturnType());
	returnValueDeclaration	.append( " ");
	returnValueDeclaration	.append(  TestNameSettings.TEST_METHOD_RETURN_VALUE_ACTUAL );
	returnValueDeclaration	.append( " = ");
	returnValueDeclaration	.append( SBFieldInit.init(method.getReturnType()));
	returnValueDeclaration	.append( ";");

	returnValueAssignment	.append( TestNameSettings.TEST_METHOD_RETURN_VALUE_ACTUAL );
	returnValueAssignment	.append( "= ");
  }

  StringBuffer fixtureInstance	= new StringBuffer();
  if( isStatic ) {
  	fixtureInstance.append( method.getClassName() );
  } else {
  	fixtureInstance.append( TestNameSettings.FIELD_SUT_INSTANCE );
  }
  fixtureInstance.append( "." );

  StringBuffer parameters = new StringBuffer();
  if( pb.hasParameters() ) {
      parameters.append( pb.buildTypelessSignature() );
  }

  StringBuffer invoke=new StringBuffer();
  if(isPrivate) {
  	invoke.append("Method method = ReflectionHelper.getMethod(");
  	invoke.append( method.getClassName() );
	invoke.append( ".class,\"");
	invoke.append( method.getMethodName() );
	invoke.append( "\",new Class[]{");
	invoke.append( NL );
	invoke.append( pb.buildClassSignature());
    invoke.append( NL );
	invoke.append( "} );" );
	invoke.append( NL );
    invoke.append( "method.setAccessible(true);");
    invoke.append( NL );
    invoke.append( returnValueAssignment );
    invoke.append( "(" );
    invoke.append( TypeWrapper.wrap( method.getReturnType() )  );
    invoke.append( ") " );
    invoke.append( "method.invoke(");
    invoke.append( TestNameSettings.FIELD_SUT_INSTANCE );
    invoke.append( "," );
    invoke.append( pb.buildTypelessSignature() );
    invoke.append( ");");
    invoke.append( NL );
  } else {
  	invoke.append( returnValueAssignment );
  	invoke.append( fixtureInstance );
  	invoke.append( method.getMethodName() );
  	invoke.append( "(" );
  	invoke.append( parameters );
  	invoke.append( ")" );
  	invoke.append( ";" );
  }

    
  // ************************************************************
  // *    now finally the real template
  // ************************************************************

    
if( catchExceptions ) {

    stringBuffer.append(TEXT_1);
    stringBuffer.append(TestNameSettings.TEST_METHOD_EXCEPTION_ACTUAL);
    stringBuffer.append(TEXT_2);
    stringBuffer.append(TEXT_3);
    stringBuffer.append(returnValueDeclaration);
    stringBuffer.append(TEXT_4);
    stringBuffer.append(invoke);
    stringBuffer.append(TEXT_5);
    stringBuffer.append(TestNameSettings.TEST_METHOD_EXCEPTION_ACTUAL);
    stringBuffer.append(TEXT_6);
    stringBuffer.append(TestNameSettings.FIELD_TEST_CONTEXT);
    stringBuffer.append(TEXT_7);
    stringBuffer.append(TestNameSettings.FIELD_TEST_CONTEXT);
    stringBuffer.append(TEXT_8);
    stringBuffer.append(TestNameSettings.FIELD_TEST_CONTEXT);
    stringBuffer.append(TEXT_9);
    
} else { // catchExceptions

    stringBuffer.append(TEXT_10);
    stringBuffer.append(returnValueDeclaration);
    stringBuffer.append(TEXT_11);
    stringBuffer.append(invoke);
    
} // catchExceptions

    return stringBuffer.toString();
  }
}