package com.seitenbau.testing.testgenerator.datadriven;

import java.util.ArrayList;
import java.util.List;

import com.seitenbau.testing.shared.config.IXmlFilterConfiguration;
import com.seitenbau.testing.shared.config.TestConfiguration;
import com.seitenbau.testing.shared.helper.FileHelper;
import com.seitenbau.testing.shared.locator.ITestPathFileBasedLocator;
import com.seitenbau.testing.shared.model.RecordedMethodConfiguration;
import com.seitenbau.testing.shared.model.RecordedTest;
import com.seitenbau.testing.shared.model.RecordedTestCase;
import com.seitenbau.testing.shared.templateengine.TemplateEngine;
import com.seitenbau.testing.shared.writer.LoggerWriter;
import com.seitenbau.testing.templates.JUnit4_DataDrivenTestGenerator;
import com.seitenbau.testing.templates.JUnit4_DataDrivenTestXmlGenerator;
import com.seitenbau.testing.testgenerator.generator.ITestGenerator;
import com.seitenbau.testing.testgenerator.generator.PerClassGenerator;
import com.seitenbau.testing.testgenerator.partgenerator.IJUnitPartGenerator;
import com.thoughtworks.xstream.XStream;

public class GeneratorDataDriven extends PerClassGenerator implements
    ITestGenerator
{

  private List<IJUnitPartGenerator> myPartGenerators = new ArrayList<IJUnitPartGenerator>();

  @Override 
  protected void createTestArtifactForClassMethods(
      TestConfiguration testConfig, List<RecordedMethodConfiguration> config,
      ITestPathFileBasedLocator pathLocator)
  {

    GeneratorDataDrivenController params = new GeneratorDataDrivenController(config,
        pathLocator, getPartGenerators());

    params.setTestPrefix(testConfig.getBaseName());

    TemplateEngine engine = new TemplateEngine();
    engine.generateToFile(new JUnit4_DataDrivenTestGenerator(), params);
 
    for (RecordedMethodConfiguration currentMethod : config)
    {
      DataDrivenXmlController param = new DataDrivenXmlController(config,
          pathLocator, getPartGenerators(), currentMethod);
      param.setTestPrefix( testConfig.getBaseName());
      engine.generateToFile(new JUnit4_DataDrivenTestXmlGenerator(), param);
    }

    createFilterFiles(testConfig, config, pathLocator);

    dumpDatabase(testConfig, config, pathLocator);

    params.printHelp(new LoggerWriter());
  }


  private void dumpDatabase(TestConfiguration testConfig,
      List<RecordedMethodConfiguration> config,
      ITestPathFileBasedLocator pathLocator)
  {

//    XStream xs = new XStream();
//    xs.registerConverter(new XStreamObjectSnapshotSweetener());
    for (RecordedMethodConfiguration cfg : config)
    {
      for (RecordedTestCase aCase : cfg.getRecordedTestCases())
      {
        for (RecordedTest test : aCase.getRecordedTests())
        {
          int testid = test.getId();
          String file = pathLocator.getTestResourceFile("dump", testid,
              "prestate.xml");
          if(test.getPreState()!=null) {
        	  String content = test.getPreState().getXMLString();
        	  FileHelper.writeFile(file, content);
          }
        }
      }
    }
  }

  private void createFilterFiles(TestConfiguration testConfig,
      List<RecordedMethodConfiguration> config,
      ITestPathFileBasedLocator pathLocator)
  {
    List<String> filterFiles = new ArrayList<String>();

    int i = 0;
    for (IXmlFilterConfiguration filter : testConfig.getFilters())
    {
      String name = filter.getClass().getSimpleName();
      String xslFile = pathLocator.getTestResourceFile("filter", i, name
          + ".xsl");
      String filenameOnly = pathLocator.getPluginFileNameOnly("filter", i, name
          + ".xsl");
      filter.saveXSLfile(xslFile);
      filterFiles.add(filenameOnly);
    }

    String file = pathLocator.getTestResourceFile("filter", "filter.xml");
    FileHelper.writeFile(file, new XStream().toXML(filterFiles));

//    String fileExculdes = pathLocator.getTestResourceFile("filter", "exclude.xml");
//    List<ExcludeFieldConfiguration> fields = config.get(0).getExcludeFields();
//    FileHelper.writeFile(fileExculdes, new XStream().toXML(fields));
  }

  public void addPartGenerator(IJUnitPartGenerator partGenerator)
  {
    getPartGenerators().add(partGenerator);
  }

  public List<IJUnitPartGenerator> getPartGenerators()
  {
    return myPartGenerators;
  }

}
