package com.seitenbau.testing.testgenerator.datadriven;

import java.util.List;

import com.seitenbau.testing.instrumentation.generator.impl.IPrintHelpMessage;
import com.seitenbau.testing.shared.locator.ITestPathFileBasedLocator;
import com.seitenbau.testing.shared.model.RecordedMethodConfiguration;
import com.seitenbau.testing.shared.templateengine.ITemplateFileParams;
import com.seitenbau.testing.shared.templateengine.TemplateFileParams;
import com.seitenbau.testing.shared.writer.IWriter;
import com.seitenbau.testing.shared.writer.ImportCollector;
import com.seitenbau.testing.shared.writer.SimplePartWriter;
import com.seitenbau.testing.testgenerator.partgenerator.IInitPartGenerator;
import com.seitenbau.testing.testgenerator.partgenerator.IJUnitPartGenerator;


public class GeneratorDataDrivenController extends TemplateFileParams implements
    ITemplateFileParams
{

  private List<RecordedMethodConfiguration> myRecordedMethodConfigurations;

  private ITestPathFileBasedLocator myPathLocator;

  private List<IJUnitPartGenerator> myPartGenerators;

  private String myTestPrefix;

  public static final String TARGET_CLASSNAME = "target-classname";

  public static final String TARGET_PACKAGE = "target-package";

  public static final String TARGET_METHODNAME = "target-methodname";

  private static final String NL = System.getProperties().getProperty(
      "line.separator");

  public GeneratorDataDrivenController(
      List<RecordedMethodConfiguration> config,
      ITestPathFileBasedLocator pathLocator,
      List<IJUnitPartGenerator> partGenerators)
  {
    myRecordedMethodConfigurations = config;
    myPathLocator = pathLocator;
    myPartGenerators = partGenerators;

    for (IJUnitPartGenerator generator : getPartGenerators())
    {
      if (generator instanceof IInitPartGenerator)
      {
        ((IInitPartGenerator) generator).init(config, pathLocator);
      }
    }
  }

  public StringBuffer includeImports()
  {
    ImportCollector collector = new ImportCollector();
    for (IJUnitPartGenerator generator : getPartGenerators())
    {
      collector.addImports(generator.getImports());
    }

    StringBuffer buffer = new StringBuffer();
    for (String oneLine : collector.getImportList())
    {
      buffer.append("import ");
      buffer.append(oneLine);
      buffer.append(";");
      buffer.append(NL);
    }
    return buffer;
  }

  public StringBuffer includeAnnotationTest()
  {
    SimplePartWriter writer = new SimplePartWriter();
    for (IJUnitPartGenerator generator : getPartGenerators())
    {
      generator.templateAnnotationTest(writer, null);
    }
    return writer.getBuffer();
  }

  public StringBuffer includeAnnotationClass()
  {
    SimplePartWriter writer = new SimplePartWriter();
    for (IJUnitPartGenerator generator : getPartGenerators())
    {
      generator.templateAnnotationClass(writer, null);
    }
    return writer.getBuffer();
  }

  public StringBuffer includeAnnotationFixture()
  {
    SimplePartWriter writer = new SimplePartWriter();
    for (IJUnitPartGenerator generator : getPartGenerators())
    {
      generator.templateAnnotationFixture(writer, null);
    }
    return writer.getBuffer();
  }

  public StringBuffer includeVariables()
  {
    SimplePartWriter writer = new SimplePartWriter();
    for (IJUnitPartGenerator generator : getPartGenerators())
    {
      generator.templateVariableContent(writer, null);
    }
    return writer.getBuffer();
  }

  public StringBuffer includeBeforeMethod()
  {
    SimplePartWriter writer = new SimplePartWriter();
    for (IJUnitPartGenerator generator : getPartGenerators())
    {
      generator.templateBeforeContent(writer, null);
    }
    return writer.getBuffer();
  }

  public StringBuffer includeBeforeTest()
  {
    SimplePartWriter writer = new SimplePartWriter();
    for (IJUnitPartGenerator generator : getPartGenerators())
    {
      generator.templateBeforeTestInvocationContent(writer, null);
    }
    return writer.getBuffer();
  }

  public StringBuffer includeAfterTest()
  {
    SimplePartWriter writer = new SimplePartWriter();
    for (IJUnitPartGenerator generator : getPartGenerators())
    {
      generator.templateAfterTestInvocationContent(writer, null);
    }
    return writer.getBuffer();
  }

  public StringBuffer includeAfterMethod()
  {
    SimplePartWriter writer = new SimplePartWriter();
    for (IJUnitPartGenerator generator : getPartGenerators())
    {
      generator.templateAfterContent(writer, null);
    }
    return writer.getBuffer();
  }

  public StringBuffer includeOtherCode()
  {
    SimplePartWriter writer = new SimplePartWriter();
    for (IJUnitPartGenerator generator : getPartGenerators())
    {
      generator.templateOtherContent(writer, null);
    }
    return writer.getBuffer();
  }

  public List<RecordedMethodConfiguration> getRecordedMethodConfigurations()
  {
    return myRecordedMethodConfigurations;
  }

  public ITestPathFileBasedLocator getPathLocator()
  {
    return myPathLocator;
  }

  public List<IJUnitPartGenerator> getPartGenerators()
  {
    return myPartGenerators;
  }

  public String getTestClassName()
  {
    RecordedMethodConfiguration first = getRecordedMethodConfigurations()
        .get(0);
    String classname = first.getClassName();
    return getTestPrefix() + classname;
  }

  public String getTestPrefix()
  {
    if (myTestPrefix == null)
    {
      return "";
    }
    return myTestPrefix;
  }

  public void setTestPrefix(String testPrefix)
  {
    myTestPrefix = testPrefix;
  }

  public void printHelp(IWriter writer)
  {
    for(IJUnitPartGenerator generator: getPartGenerators())
    {
      if(generator instanceof IPrintHelpMessage)
      {
        ((IPrintHelpMessage)generator).printHelpMessage( writer );
      }
    }

  }

}
