package com.seitenbau.testing.testgenerator.generator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.seitenbau.testing.shared.config.TestConfiguration;
import com.seitenbau.testing.shared.helper.DeepCopy;
import com.seitenbau.testing.shared.helper.Generics;
import com.seitenbau.testing.shared.locator.ITestPathFileBasedLocator;
import com.seitenbau.testing.shared.locator.TestPathFileBasedLocator;
import com.seitenbau.testing.shared.model.RecordedMethodConfiguration;


abstract public class PerClassGenerator implements ITestGenerator
{

  abstract protected void createTestArtifactForClassMethods(
      TestConfiguration testConfig, List<RecordedMethodConfiguration> config,
      ITestPathFileBasedLocator pathLocator);

  public void createTestArtifacts(TestConfiguration testConfig,
      String testRootFolder, String testResourcesRootFolder)
  {
    if (testConfig == null)
    {
      throw new IllegalArgumentException("TestConfiguration was null.");
    }
    if (testConfig.getTestData() == null)
    {
      throw new IllegalArgumentException(
          "TestConfiguration RecordedMethodList was null.");
    }
    if (testConfig.getTestData().isEmpty())
    {
      throw new IllegalArgumentException(
          "TestConfiguration RecordedMethodList was empty.");
    }

    List<RecordedMethodConfiguration> newList = new ArrayList<RecordedMethodConfiguration>();
    for (RecordedMethodConfiguration cfg : testConfig.getTestData())
    {
      newList.add(DeepCopy.createCopy(cfg));
    }

    testConfig.setTestData(newList);

    List<RecordedMethodConfiguration> config = testConfig.getTestData();

    HashMap<String, List<RecordedMethodConfiguration>> groupedTests = generateGrouping(config);

    for (String key : Generics.interatorMapKeys(groupedTests))
    {
      List<RecordedMethodConfiguration> list = groupedTests.get(key);

      ITestPathFileBasedLocator pathLocator = createPathLocator(testConfig,testRootFolder,
          testResourcesRootFolder, list.get(0));

      createTestArtifactForClassMethods(testConfig, list, pathLocator);
    }

  }

  protected ITestPathFileBasedLocator createPathLocator(TestConfiguration config, String testRootFolder,
      String testResourcesRootFolder,
      RecordedMethodConfiguration recordedMethodConfiguration)
  {
    RecordedMethodConfiguration firstElement = recordedMethodConfiguration;
    String classPackage = firstElement.getClassPackage();
    String className = firstElement.getClassName();
    String prefix=config.getBaseName();

    ITestPathFileBasedLocator pathLocator = new TestPathFileBasedLocator(
        prefix,testRootFolder, testResourcesRootFolder, classPackage, className);
    return pathLocator;
  }

  private HashMap<String, List<RecordedMethodConfiguration>> generateGrouping(
      List<RecordedMethodConfiguration> recordedMethodConfiguration)
  {
    HashMap<String, List<RecordedMethodConfiguration>> grouped = new HashMap<String, List<RecordedMethodConfiguration>>();
    for (RecordedMethodConfiguration rmConfig : recordedMethodConfiguration)
    {
      String key = rmConfig.getClassPackage() + "." + rmConfig.getClassName();
      List<RecordedMethodConfiguration> list = grouped.get(key);
      if (list == null)
      {
        list = new ArrayList<RecordedMethodConfiguration>();
        grouped.put(key, list);
      }
      list.add(rmConfig);
    }
    return grouped;
  }
}
