package com.seitenbau.testing.testgenerator.partgenerator.velocity;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.List;

import org.apache.velocity.context.Context;

import com.seitenbau.testing.instrumentation.generator.impl.IPrintHelpMessage;
import com.seitenbau.testing.shared.config.TestConfiguration;
import com.seitenbau.testing.shared.helper.ParameterBuilder;
import com.seitenbau.testing.shared.locator.ITestPathFileBasedLocator;
import com.seitenbau.testing.shared.model.RecordedMethodConfiguration;
import com.seitenbau.testing.shared.model.RecordedTest;
import com.seitenbau.testing.shared.model.RecordedTestCase;
import com.seitenbau.testing.shared.writer.IPartWriter;
import com.seitenbau.testing.shared.writer.IWriter;
import com.seitenbau.testing.shared.writer.SimplePartWriter;
import com.seitenbau.testing.testgenerator.TestNameSettings;
import com.seitenbau.testing.testgenerator.partgenerator.IInitPartGenerator;
import com.seitenbau.testing.testgenerator.partgenerator.IJUnitPartGenerator;

/**
 * Implementation for the JUnitPartGenerator while the contents are
 * read from a Velocity Template
 *
 */
abstract public class AbstractTemplatePartGenerator extends VelocityTemplate
    implements IJUnitPartGenerator, IInitPartGenerator,IPrintHelpMessage
{

  private static final String TEMPLATE_ID_ANNOTATION_CLASS = "annotationClass";

  private static final String TEMPLATE_ID_ANNOTATION_FIXTURE = "annotationFixture";

  private static final String TEMPLATE_ID_TEST_ANNOTATION = "annotationTest";

  private static final String TEMPLATE_ID_VARIABLES = "variables";

  private static final String TEMPLATE_ID_BEFORE = "before";

  private static final String TEMPLATE_ID_BEFORE_TEST_INVOCATION = "beforetest";

  private static final String TEMPLATE_ID_AFTER_TEST_INVOCATION = "aftertest";

  private static final String TEMPLATE_ID_AFTER = "after";

  private static final String TEMPLATE_ID_OTHER = "other";

  private static final String TEMPLATE_ID_HELP_TEXT = "help";

  private List<RecordedMethodConfiguration> myRecordedConfigurations;

  private ITestPathFileBasedLocator myPathLocator;

  @Override
  protected String getTemplateFileName()
  {
    String clazzName = getClass().getSimpleName();
    return clazzName + ".template";
  }

  @Override
  protected void prepareContext(Context context)
  {
    RecordedMethodConfiguration justone = getFirstRecordedMethod();

    context.put("target-classname", justone.getClassName());
    context.put("target-package", justone.getClassPackage());
    context.put("target-methodname", justone.getMethodName());
    context.put("target", justone);

    RecordedTest justoneTest = getAnRecordedTest();
    context.put("target-isPrivate", Modifier.isPrivate(justoneTest
        .getModifiers()));
    context.put("target-isStatic", Modifier
        .isStatic(justoneTest.getModifiers()));
    context.put("target-isVoid", justoneTest.isVoidMethod());

    String params = justone.getParameters();
    context.put("parameterTypes", new ParameterBuilder(params).getAsList());

    addConstants(TestNameSettings.class, context);

    super.prepareContext(context);
  }

  private void addConstants(Class<?> clazz, Context context)
  {
    Field[] fields = clazz.getDeclaredFields();
    for (Field field : fields)
    {
      if (Modifier.isStatic(field.getModifiers()))
      {
        Object value;
        try
        {
          value = field.get(null);
          String name = field.getName();
          context.put(name, value);
        }
        catch (IllegalArgumentException e)
        {
        }
        catch (IllegalAccessException e)
        {
        }
      }
    }
  }

  protected RecordedTest getAnRecordedTest()
  {
    RecordedTestCase anTestCase = getFirstRecordedMethod()
        .getRecordedTestCases().get(0);

     List<RecordedMethodConfiguration> list = getRecordedConfigurations();
     for(RecordedMethodConfiguration cfg:list) {
       if( cfg.getRecordedTestCases()!=null ) {
         for(RecordedTestCase testCase:cfg.getRecordedTestCases()) {
           if(testCase.getRecordedTests()!=null) {
             for(RecordedTest test:testCase.getRecordedTests()) {
               if(test!=null) {
                 return test;
               }
             }
           }
         }
       }
     }
    List<RecordedTest> tests = anTestCase.getRecordedTests();
    // for( RecordedTest test:tests) {
    // System.out.println(test);
    // }
    return tests.get(0);
  }

  protected RecordedMethodConfiguration getFirstRecordedMethod()
  {
    return getRecordedConfigurations().get(0);
  }

  public List<String> getImports()
  {
    return null;
  }

  public void init(List<RecordedMethodConfiguration> config,
      ITestPathFileBasedLocator pathLocator)
  {
    myRecordedConfigurations = config;
    myPathLocator = pathLocator;

    initTemplateStore();
  }

  public void templateAfterContent(IPartWriter writer, TestConfiguration config)
  {
    template(TEMPLATE_ID_AFTER, writer, config);
  }

  protected void template(String id, IPartWriter writer,
      TestConfiguration config)
  {
    StringBuffer cont = getContent(id, "");
    writer.append(cont);
  }

  public void templateAfterTestInvocationContent(IPartWriter writer,
      TestConfiguration config)
  {
    template(TEMPLATE_ID_AFTER_TEST_INVOCATION, writer, config);
  }

  public void templateAnnotationTest(SimplePartWriter writer,
      TestConfiguration config)
  {
    template(TEMPLATE_ID_TEST_ANNOTATION, writer, config);
  }

  public void templateBeforeContent(IPartWriter writer, TestConfiguration config)
  {
    template(TEMPLATE_ID_BEFORE, writer, config);
  }

  public void templateBeforeTestInvocationContent(IPartWriter writer,
      TestConfiguration config)
  {
    template(TEMPLATE_ID_BEFORE_TEST_INVOCATION, writer, config);
  }

  public void templateVariableContent(IPartWriter writer,
      TestConfiguration config)
  {
    template(TEMPLATE_ID_VARIABLES, writer, config);
  }

  public void templateAnnotationClass(IPartWriter writer,
      TestConfiguration config)
  {
    template(TEMPLATE_ID_ANNOTATION_CLASS, writer, config);
  }

  public void templateAnnotationFixture(IPartWriter writer,
      TestConfiguration config)
  {
    template(TEMPLATE_ID_ANNOTATION_FIXTURE, writer, config);
  }

  public void templateOtherContent(IPartWriter writer, TestConfiguration config)
  {
    template(TEMPLATE_ID_OTHER, writer, config);
  }

  public List<RecordedMethodConfiguration> getRecordedConfigurations()
  {
    return myRecordedConfigurations;
  }

  public void setRecordedConfigurations(
      List<RecordedMethodConfiguration> recordedConfigurations)
  {
    myRecordedConfigurations = recordedConfigurations;
  }

  public ITestPathFileBasedLocator getPathLocator()
  {
    return myPathLocator;
  }

  public void setPathLocator(ITestPathFileBasedLocator pathLocator)
  {
    myPathLocator = pathLocator;
  }

  public void printHelpMessage(IWriter writer)
  {
    StringBuffer cont = getContent(TEMPLATE_ID_HELP_TEXT, "");
    writer.append(cont);
    writer.appendNewLine();
  }

}
